@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: avx512.cc 13716 2019-12-21 20:07:03Z sshwarts $
+// $Id: avx512.cc 13853 2020-05-19 16:01:23Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2013-2019 Stanislav Shwartsman
@@ -30,6 +30,7 @@
 
 #include "simd_int.h"
 #include "simd_compare.h"
+#include "wide_int.h"
 
 // compare
 
@@ -2272,8 +2273,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VPMULTISHIFTQB_MASK_VdqHdqWdqR(bxInstructi
 
 // 52-bit integer FMA
 
-extern void long_mul(Bit128u *product, Bit64u op1, Bit64u op2);
-
 BX_CPP_INLINE Bit64u pmadd52luq_scalar(Bit64u dst, Bit64u op1, Bit64u op2)
 {
   op1 &= BX_CONST64(0x000fffffffffffff);
