@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: generic_cpuid.cc 13657 2019-12-09 18:16:29Z sshwarts $
+// $Id: generic_cpuid.cc 14149 2021-02-16 18:57:49Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2011-2017 Stanislav Shwartsman
@@ -22,6 +22,7 @@
 /////////////////////////////////////////////////////////////////////////
 
 #include "bochs.h"
+#include "gui/siminterface.h"
 #include "cpu.h"
 #include "param_names.h"
 #include "generic_cpuid.h"
@@ -55,19 +56,16 @@ bx_generic_cpuid_t::bx_generic_cpuid_t(BX_CPU_C *cpu): bx_cpuid_t(cpu)
 
   // do not report CPUID functions above 0x3 if cpuid_limit_winnt is set
   // to workaround WinNT issue.
-  static bx_bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
-  if (! cpuid_limit_winnt) {
-    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_MONITOR_MWAIT))
-      max_std_leaf = 0x5;
-    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_X2APIC))
-      max_std_leaf = 0xB;
-    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_XSAVE))
-      max_std_leaf = 0xD;
-  }
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_MONITOR_MWAIT))
+    max_std_leaf = 0x5;
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_X2APIC))
+    max_std_leaf = 0xB;
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_XSAVE))
+    max_std_leaf = 0xD;
 #endif
 
 #if BX_CPU_LEVEL <= 5
-  max_ext_leaf = 0;
+  max_ext_leaf = 0x0;
 #else
   max_ext_leaf = 0x80000008;
 
@@ -80,7 +78,7 @@ void bx_generic_cpuid_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpu
 {
   static const char *brand_string = (const char *)SIM->get_param_string(BXPN_BRAND_STRING)->getptr();
 
-  static bx_bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
+  static bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
   if (cpuid_limit_winnt)
     if (function > 2 && function < 0x80000000) function = 2;
 
@@ -177,12 +175,7 @@ void bx_generic_cpuid_t::get_std_cpuid_leaf_0(cpuid_function_t *leaf) const
   // EBX: vendor ID string
   // EDX: vendor ID string
   // ECX: vendor ID string
-  unsigned max_leaf = max_std_leaf;
-  static bx_bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
-  if (cpuid_limit_winnt)
-    max_leaf = 0x2;
-
-  get_leaf_0(max_leaf, (const char *) vendor_string, leaf);
+  get_leaf_0(max_std_leaf, (const char *) vendor_string, leaf);
 }
 
 // leaf 0x00000001 //
@@ -302,10 +295,18 @@ void bx_generic_cpuid_t::get_std_cpuid_leaf_6(cpuid_function_t *leaf) const
 // leaf 0x00000007 //
 void bx_generic_cpuid_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_function_t *leaf) const
 {
-  leaf->eax = 0; /* report max sub-leaf that supported in leaf 7 */
-  leaf->ebx = get_ext3_cpuid_features();
-  leaf->ecx = get_ext4_cpuid_features();
-  leaf->edx = 0;
+  switch(subfunction) {
+  case 0:
+    leaf->eax = 0; /* report max sub-leaf that supported in leaf 7 */
+    leaf->ebx = get_std_cpuid_leaf_7_ebx(BX_CPU_VENDOR_INTEL ? BX_CPUID_EXT3_ENCHANCED_REP_STRINGS : 0);
+    leaf->ecx = get_std_cpuid_leaf_7_ecx();
+    leaf->edx = 0;
+  default:
+    leaf->eax = 0;
+    leaf->ebx = 0;
+    leaf->ecx = 0;
+    leaf->edx = 0;
+  }
 }
 
 // leaf 0x00000008 reserved                          //
@@ -507,7 +508,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
   enable_cpu_extension(BX_ISA_486);
 
 #if BX_CPU_LEVEL >= 5
-  static bx_bool mmx_enabled = SIM->get_param_bool(BXPN_CPUID_MMX)->get();
+  static bool mmx_enabled = SIM->get_param_bool(BXPN_CPUID_MMX)->get();
 
   if (cpu_level >= 5) {
     enable_cpu_extension(BX_ISA_PENTIUM);
@@ -571,7 +572,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
   }
 
 #if BX_SUPPORT_MONITOR_MWAIT
-  static bx_bool mwait_enabled = SIM->get_param_bool(BXPN_CPUID_MWAIT)->get();
+  static bool mwait_enabled = SIM->get_param_bool(BXPN_CPUID_MWAIT)->get();
   if (mwait_enabled) {
     enable_cpu_extension(BX_ISA_MONITOR_MWAIT);
     if (cpu_level < 6)
@@ -628,7 +629,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
   if (simd_enabled >= BX_CPUID_SUPPORT_SSE4_2)
     enable_cpu_extension(BX_ISA_POPCNT);
 
-  static bx_bool sse4a_enabled = SIM->get_param_bool(BXPN_CPUID_SSE4A)->get();
+  static bool sse4a_enabled = SIM->get_param_bool(BXPN_CPUID_SSE4A)->get();
   if (sse4a_enabled) {
     enable_cpu_extension(BX_ISA_SSE4A);
 
@@ -638,7 +639,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool misaligned_sse_enabled = SIM->get_param_bool(BXPN_CPUID_MISALIGNED_SSE)->get();
+  static bool misaligned_sse_enabled = SIM->get_param_bool(BXPN_CPUID_MISALIGNED_SSE)->get();
   if (misaligned_sse_enabled) {
     enable_cpu_extension(BX_ISA_MISALIGNED_SSE);
     if (cpu_level < 6)
@@ -650,7 +651,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool sep_enabled = SIM->get_param_bool(BXPN_CPUID_SEP)->get();
+  static bool sep_enabled = SIM->get_param_bool(BXPN_CPUID_SEP)->get();
   if (sep_enabled) {
     enable_cpu_extension(BX_ISA_SYSENTER_SYSEXIT);
     if (cpu_level < 6) {
@@ -659,7 +660,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool xsave_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVE)->get();
+  static bool xsave_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVE)->get();
   if (xsave_enabled) {
     enable_cpu_extension(BX_ISA_XSAVE);
 
@@ -669,7 +670,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool xsaveopt_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVEOPT)->get();
+  static bool xsaveopt_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVEOPT)->get();
   if (xsaveopt_enabled) {
     enable_cpu_extension(BX_ISA_XSAVEOPT);
 
@@ -679,7 +680,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool aes_enabled = SIM->get_param_bool(BXPN_CPUID_AES)->get();
+  static bool aes_enabled = SIM->get_param_bool(BXPN_CPUID_AES)->get();
   if (aes_enabled) {
     enable_cpu_extension(BX_ISA_AES_PCLMULQDQ);
 
@@ -690,7 +691,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool sha_enabled = SIM->get_param_bool(BXPN_CPUID_SHA)->get();
+  static bool sha_enabled = SIM->get_param_bool(BXPN_CPUID_SHA)->get();
   if (sha_enabled) {
     enable_cpu_extension(BX_ISA_SHA);
 
@@ -701,7 +702,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool movbe_enabled = SIM->get_param_bool(BXPN_CPUID_MOVBE)->get();
+  static bool movbe_enabled = SIM->get_param_bool(BXPN_CPUID_MOVBE)->get();
   if (movbe_enabled) {
     enable_cpu_extension(BX_ISA_MOVBE);
 
@@ -712,7 +713,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool adx_enabled = SIM->get_param_bool(BXPN_CPUID_ADX)->get();
+  static bool adx_enabled = SIM->get_param_bool(BXPN_CPUID_ADX)->get();
   if (adx_enabled) {
     enable_cpu_extension(BX_ISA_ADX);
 
@@ -724,7 +725,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
   }
 
 #if BX_SUPPORT_X86_64
-  static bx_bool x86_64_enabled = SIM->get_param_bool(BXPN_CPUID_X86_64)->get();
+  static bool x86_64_enabled = SIM->get_param_bool(BXPN_CPUID_X86_64)->get();
   if (x86_64_enabled) {
     if (cpu_level < 6) {
       BX_PANIC(("PANIC: x86-64 emulation requires P6 CPU level support !"));
@@ -749,7 +750,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
       return;
     }
 
-    static bx_bool fsgsbase_enabled = SIM->get_param_bool(BXPN_CPUID_FSGSBASE)->get();
+    static bool fsgsbase_enabled = SIM->get_param_bool(BXPN_CPUID_FSGSBASE)->get();
     if (fsgsbase_enabled)
       enable_cpu_extension(BX_ISA_FSGSBASE);
 
@@ -759,11 +760,11 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
       return;
     }
 
-    static bx_bool pcid_enabled = SIM->get_param_bool(BXPN_CPUID_PCID)->get();
+    static bool pcid_enabled = SIM->get_param_bool(BXPN_CPUID_PCID)->get();
     if (pcid_enabled)
       enable_cpu_extension(BX_ISA_PCID);
 
-    static bx_bool xlarge_pages = SIM->get_param_bool(BXPN_CPUID_1G_PAGES)->get();
+    static bool xlarge_pages = SIM->get_param_bool(BXPN_CPUID_1G_PAGES)->get();
     if (xlarge_pages)
       enable_cpu_extension(BX_ISA_1G_PAGES);
   }
@@ -788,7 +789,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     }
   }
 
-  static bx_bool avx_f16c_enabled = SIM->get_param_bool(BXPN_CPUID_AVX_F16CVT)->get();
+  static bool avx_f16c_enabled = SIM->get_param_bool(BXPN_CPUID_AVX_F16CVT)->get();
   if (avx_f16c_enabled) {
     if (simd_enabled < BX_CPUID_SUPPORT_AVX) {
       BX_PANIC(("PANIC: Float16 convert emulation requires AVX support !"));
@@ -798,7 +799,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     enable_cpu_extension(BX_ISA_AVX_F16C);
   }
 
-  static bx_bool avx_fma_enabled = SIM->get_param_bool(BXPN_CPUID_AVX_FMA)->get();
+  static bool avx_fma_enabled = SIM->get_param_bool(BXPN_CPUID_AVX_FMA)->get();
   if (avx_fma_enabled) {
     if (simd_enabled < BX_CPUID_SUPPORT_AVX2) {
       BX_PANIC(("PANIC: FMA emulation requires AVX2 support !"));
@@ -822,7 +823,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
       enable_cpu_extension(BX_ISA_BMI2);
   }
 
-  static bx_bool fma4_enabled = SIM->get_param_bool(BXPN_CPUID_FMA4)->get();
+  static bool fma4_enabled = SIM->get_param_bool(BXPN_CPUID_FMA4)->get();
   if (fma4_enabled) {
     if (simd_enabled < BX_CPUID_SUPPORT_AVX) {
       BX_PANIC(("PANIC: FMA4 emulation requires AVX support !"));
@@ -832,7 +833,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     enable_cpu_extension(BX_ISA_FMA4);
   }
 
-  static bx_bool xop_enabled = SIM->get_param_bool(BXPN_CPUID_XOP)->get();
+  static bool xop_enabled = SIM->get_param_bool(BXPN_CPUID_XOP)->get();
   if (xop_enabled) {
     if (simd_enabled < BX_CPUID_SUPPORT_AVX) {
       BX_PANIC(("PANIC: XOP emulation requires AVX support !"));
@@ -842,7 +843,7 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
     enable_cpu_extension(BX_ISA_XOP);
   }
 
-  static bx_bool tbm_enabled = SIM->get_param_bool(BXPN_CPUID_TBM)->get();
+  static bool tbm_enabled = SIM->get_param_bool(BXPN_CPUID_TBM)->get();
   if (tbm_enabled) {
     if (simd_enabled < BX_CPUID_SUPPORT_AVX || ! xop_enabled) {
       BX_PANIC(("PANIC: TBM emulation requires AVX and XOP support !"));
@@ -890,14 +891,14 @@ void bx_generic_cpuid_t::init_cpu_extensions_bitmask(void)
   }
 #endif
 
-  static bx_bool smep_enabled = SIM->get_param_bool(BXPN_CPUID_SMEP)->get();
+  static bool smep_enabled = SIM->get_param_bool(BXPN_CPUID_SMEP)->get();
   if (smep_enabled) {
     enable_cpu_extension(BX_ISA_SMEP);
     if (cpu_level < 6)
       BX_PANIC(("PANIC: SMEP emulation requires P6 CPU level support !"));
   }
 
-  static bx_bool smap_enabled = SIM->get_param_bool(BXPN_CPUID_SMAP)->get();
+  static bool smap_enabled = SIM->get_param_bool(BXPN_CPUID_SMAP)->get();
   if (smap_enabled) {
     enable_cpu_extension(BX_ISA_SMAP);
     if (cpu_level < 6)
@@ -921,7 +922,7 @@ void bx_generic_cpuid_t::init_vmx_extensions_bitmask(void)
     features_bitmask |= BX_VMX_VIRTUAL_NMI;
 
 #if BX_SUPPORT_X86_64
-    static bx_bool x86_64_enabled = SIM->get_param_bool(BXPN_CPUID_X86_64)->get();
+    static bool x86_64_enabled = SIM->get_param_bool(BXPN_CPUID_X86_64)->get();
     if (x86_64_enabled) {
       features_bitmask |= BX_VMX_TPR_SHADOW |
                           BX_VMX_APIC_VIRTUALIZATION |
@@ -961,7 +962,7 @@ void bx_generic_cpuid_t::init_svm_extensions_bitmask(void)
 {
   Bit32u features_bitmask = 0;
 
-  static bx_bool svm_enabled = SIM->get_param_bool(BXPN_CPUID_SVM)->get();
+  static bool svm_enabled = SIM->get_param_bool(BXPN_CPUID_SVM)->get();
   if (svm_enabled) {
     features_bitmask = BX_CPUID_SVM_NESTED_PAGING |
                        BX_CPUID_SVM_NRIP_SAVE;
@@ -1213,7 +1214,9 @@ Bit32u bx_generic_cpuid_t::get_std_cpuid_features(void) const
     features |= BX_CPUID_STD_SELF_SNOOP;
 #endif
 
+#if BX_SUPPORT_SMP
   features |= BX_CPUID_STD_HT;
+#endif
 
   return features;
 }
@@ -1338,141 +1341,6 @@ Bit32u bx_generic_cpuid_t::get_ext2_cpuid_features(void) const
   return features;
 }
 
-Bit32u bx_generic_cpuid_t::get_ext3_cpuid_features(void) const
-{
-  Bit32u features = 0;
-
-  //   [0:0]    FS/GS BASE access instructions
-  //   [1:1]    Support for IA32_TSC_ADJUST MSR
-  //   [2:2]    reserved
-  //   [3:3]    BMI1: Advanced Bit Manipulation Extensions
-  //   [4:4]    HLE: Hardware Lock Elision
-  //   [5:5]    AVX2
-  //   [6:6]    reserved
-  //   [7:7]    SMEP: Supervisor Mode Execution Protection
-  //   [8:8]    BMI2: Advanced Bit Manipulation Extensions
-  //   [9:9]    Support for Enhanced REP MOVSB/STOSB
-  //   [10:10]  Support for INVPCID instruction
-  //   [11:11]  RTM: Restricted Transactional Memory
-  //   [12:12]  Supports Quality of Service (QoS) capability
-  //   [13:13]  Deprecates FPU CS and FPU DS values
-  //   [14:14]  Intel Memory Protection Extensions
-  //   [15:15]  reserved
-  //   [16:16]  AVX512F instructions support
-  //   [17:17]  reserved
-  //   [18:18]  RDSEED instruction support
-  //   [19:19]  ADCX/ADOX instructions support
-  //   [20:20]  SMAP: Supervisor Mode Access Prevention
-  //   [24:21]  reserved
-  //   [25:25]  Intel Processor Trace
-  //   [26:26]  AVX512PF instructions support
-  //   [27:27]  AVX512ER instructions support
-  //   [28:28]  AVX512CD instructions support
-  //   [29:29]  SHA instructions support
-  //   [31:30]  reserved
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_FSGSBASE))
-    features |= BX_CPUID_EXT3_FSGSBASE;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_BMI1))
-    features |= BX_CPUID_EXT3_BMI1;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX2))
-    features |= BX_CPUID_EXT3_AVX2;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SMEP))
-    features |= BX_CPUID_EXT3_SMEP;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_BMI2))
-    features |= BX_CPUID_EXT3_BMI2;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_INVPCID))
-    features |= BX_CPUID_EXT3_INVPCID;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_FCS_FDS_DEPRECATION))
-    features |= BX_CPUID_EXT3_DEPRECATE_FCS_FDS;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512))
-    features |= BX_CPUID_EXT3_AVX512F;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_RDSEED))
-    features |= BX_CPUID_EXT3_RDSEED;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_ADX))
-    features |= BX_CPUID_EXT3_ADX;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SMAP))
-    features |= BX_CPUID_EXT3_SMAP;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SHA))
-    features |= BX_CPUID_EXT3_SHA;
-
-  return features;
-}
-
-Bit32u bx_generic_cpuid_t::get_ext4_cpuid_features(void) const
-{
-  Bit32u features = 0;
-
-  //   [0:0] PREFETCHWT1 instruction support
-  //   [1:1] AVX512 VBMI instructions support
-  //   [2:2] UMIP: Supports user-mode instruction prevention
-  //   [3:3] PKU: Protection keys for user-mode pages.
-  //   [4:4] OSPKE: OS has set CR4.PKE to enable protection keys
-  //   [5:5] reserved
-  //   [6:6] AVX512 VBMI2 instructions support
-  //   [7:7] reserved
-  //   [8:8] GFNI instructions support
-  //   [9:9] VAES instructions support
-  // [10:10] VPCLMULQDQ instruction support
-  // [11:11] AVX512 VNNI instructions support
-  // [12:12] AVX512 BITALG instructions support
-  // [13:13] reserved
-  // [14:14] AVX512 VPOPCNTDQ: AVX512 VPOPCNTD/VPOPCNTQ instructions
-  // [15:15] reserved
-  // [16:16] LA57: LA57 and 5-level paging
-  // [21:17] reserved
-  // [22:22] RDPID: Read Processor ID support
-  // [29:23] reserved
-  // [30:30] SGX_LC: SGX Launch Configuration
-  // [31:31] reserved
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512_VBMI))
-    features |= BX_CPUID_EXT4_AVX512_VBMI;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_UMIP))
-    features |= BX_CPUID_EXT4_UMIP;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512_VBMI2))
-    features |= BX_CPUID_EXT4_AVX512_VBMI2;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_GFNI))
-    features |= BX_CPUID_EXT4_GFNI;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_VAES_VPCLMULQDQ))
-    features |= BX_CPUID_EXT4_VAES | BX_CPUID_EXT4_VPCLMULQDQ;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512_VNNI))
-    features |= BX_CPUID_EXT4_AVX512_VNNI;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512_BITALG))
-    features |= BX_CPUID_EXT4_AVX512_BITALG;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512_VPOPCNTDQ))
-    features |= BX_CPUID_EXT4_AVX512_VPOPCNTDQ;
-
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_RDPID))
-    features |= BX_CPUID_EXT4_RDPID;
-
-#if BX_SUPPORT_PKEYS
-  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_PKU)) {
-    features |= BX_CPUID_EXT4_PKU;
-    if (cpu->cr4.get_PKE())
-      features |= BX_CPUID_EXT4_OSPKE;
-  }
-#endif
-
-  return features;
-}
-
 #endif
 
 void bx_generic_cpuid_t::dump_cpuid(void) const
