@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: init.cc 13761 2020-01-03 05:29:45Z sshwarts $
+// $Id: init.cc 14320 2021-07-25 18:01:28Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2019  The Bochs Project
@@ -25,6 +25,7 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#include "gui/siminterface.h"
 #include "param_names.h"
 #include "cpustats.h"
 
@@ -69,6 +70,13 @@ BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
 
 #include "generic_cpuid.h"
 
+enum {
+#define bx_define_cpudb(model) bx_cpudb_##model,
+#include "cpudb.h"
+  bx_cpudb_model_last
+};
+#undef bx_define_cpudb
+
 #define bx_define_cpudb(model) \
   extern bx_cpuid_t *create_ ##model##_cpuid(BX_CPU_C *cpu);
 
@@ -269,7 +277,7 @@ void BX_CPU_C::register_state(void)
 #endif
 
 #if BX_CPU_LEVEL >= 5
-  BXRS_HEX_PARAM_FIELD(cpu, tsc_last_reset, tsc_last_reset);
+  BXRS_HEX_PARAM_FIELD(cpu, tsc_adjust, tsc_adjust);
 #if BX_SUPPORT_VMX || BX_SUPPORT_SVM
   BXRS_HEX_PARAM_FIELD(cpu, tsc_offset, tsc_offset);
 #endif
@@ -277,6 +285,7 @@ void BX_CPU_C::register_state(void)
 
 #if BX_SUPPORT_PKEYS
   BXRS_HEX_PARAM_FIELD(cpu, pkru, pkru);
+  BXRS_HEX_PARAM_FIELD(cpu, pkrs, pkrs);
 #endif
 
   for(n=0; n<6; n++) {
@@ -415,6 +424,9 @@ void BX_CPU_C::register_state(void)
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SCA_MITIGATIONS)) {
     BXRS_HEX_PARAM_FIELD(MSR, ia32_spec_ctrl, msr.ia32_spec_ctrl);
   }
+#if BX_SUPPORT_VMX
+  BXRS_HEX_PARAM_FIELD(MSR, ia32_feature_ctrl, msr.ia32_feature_ctrl);
+#endif
 
 #if BX_CONFIGURE_MSRS
   bx_list_c *MSRS = new bx_list_c(cpu, "USER_MSR");
@@ -652,8 +664,12 @@ void BX_CPU_C::after_restore_state(void)
   set_VMCSPTR(BX_CPU_THIS_PTR vmcsptr);
 #endif
 
+#if BX_SUPPORT_SVM
+  set_VMCBPTR(BX_CPU_THIS_PTR vmcbptr);
+#endif
+
 #if BX_SUPPORT_PKEYS
-  set_PKRU(BX_CPU_THIS_PTR pkru);
+  set_PKeys(BX_CPU_THIS_PTR pkru, BX_CPU_THIS_PTR pkrs);
 #endif
 
   assert_checks();
@@ -939,7 +955,7 @@ void BX_CPU_C::reset(unsigned source)
   if (source == BX_RESET_HARDWARE) {
 
 #if BX_SUPPORT_PKEYS
-    BX_CPU_THIS_PTR set_PKRU(0);
+    BX_CPU_THIS_PTR set_PKeys(0, 0);
 #endif
 
 #if BX_CPU_LEVEL >= 6
@@ -1046,20 +1062,14 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR vmcsptr = BX_CPU_THIS_PTR vmxonptr = BX_INVALID_VMCSPTR;
   set_VMCSPTR(BX_CPU_THIS_PTR vmcsptr);
   if (source == BX_RESET_HARDWARE) {
-    /* enable VMX, should be done in BIOS instead */
-    BX_CPU_THIS_PTR msr.ia32_feature_ctrl =
-      /*BX_IA32_FEATURE_CONTROL_LOCK_BIT | */BX_IA32_FEATURE_CONTROL_VMX_ENABLE_BIT;
+    BX_CPU_THIS_PTR msr.ia32_feature_ctrl = 0;
   }
 #endif
 
 #if BX_SUPPORT_SVM
+  set_VMCBPTR(0);
   BX_CPU_THIS_PTR in_svm_guest = 0;
   BX_CPU_THIS_PTR svm_gif = 1;
-  BX_CPU_THIS_PTR vmcbptr = 0;
-  BX_CPU_THIS_PTR vmcbhostptr = 0;
-#if BX_SUPPORT_MEMTYPE
-  BX_CPU_THIS_PTR vmcb_memtype = BX_MEMTYPE_UC;
-#endif
 #endif
 
 #if BX_SUPPORT_VMX || BX_SUPPORT_SVM
