@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.cc 13652 2019-12-09 15:18:47Z sshwarts $
+// $Id: siminterface.cc 14293 2021-06-27 14:50:26Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2018  The Bochs Project
+//  Copyright (C) 2002-2021  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 
 #include "param_names.h"
 #include "iodev.h"
+#include "bx_debug/debug.h"
 #include "virt_timer.h"
 
 bx_simulator_interface_c *SIM = NULL;
@@ -70,21 +71,21 @@ class bx_real_sim_c : public bx_simulator_interface_c {
   void *ci_callback_data;
   rt_conf_entry_t *rt_conf_entries;
   addon_option_t *addon_options;
-  int init_done;
-  int enabled;
+  bool init_done;
+  bool enabled;
   // save context to jump to if we must quit unexpectedly
   jmp_buf *quit_context;
   int exit_code;
   unsigned param_id;
-  bx_bool bx_debug_gui;
-  bx_bool bx_log_viewer;
-  bx_bool wxsel;
+  bool bx_debug_gui;
+  bool bx_log_viewer;
+  bool wxsel;
 public:
   bx_real_sim_c();
   virtual ~bx_real_sim_c() {}
   virtual void set_quit_context(jmp_buf *context) { quit_context = context; }
-  virtual int get_init_done() { return init_done; }
-  virtual int set_init_done(int n) { init_done = n; return 0;}
+  virtual bool get_init_done() { return init_done; }
+  virtual int set_init_done(bool n);
   virtual void reset_all_param();
   // new param methods
   virtual bx_param_c *get_param(const char *pname, bx_param_c *base=NULL);
@@ -120,23 +121,24 @@ public:
   virtual int set_log_prefix(const char *prefix);
   virtual int get_debugger_log_file(char *path, int len);
   virtual int set_debugger_log_file(const char *path);
-  virtual int hdimage_get_mode(const char *mode);
   virtual void set_notify_callback(bxevent_handler func, void *arg);
   virtual void get_notify_callback(bxevent_handler *func, void **arg);
   virtual BxEvent* sim_to_ci_event(BxEvent *event);
   virtual int log_dlg(const char *prefix, int level, const char *msg, int mode);
   virtual void log_msg(const char *prefix, int level, const char *msg);
-  virtual void set_log_viewer(bx_bool val) { bx_log_viewer = val; }
-  virtual bx_bool has_log_viewer() const { return bx_log_viewer; }
+  virtual void set_log_viewer(bool val) { bx_log_viewer = val; }
+  virtual bool has_log_viewer() const { return bx_log_viewer; }
   virtual int ask_param(bx_param_c *param);
   virtual int ask_param(const char *pname);
   // ask the user for a pathname
   virtual int ask_filename(const char *filename, int maxlen, const char *prompt, const char *the_default, int flags);
   // yes/no dialog
-  virtual int ask_yes_no(const char *title, const char *prompt, bx_bool the_default);
+  virtual int ask_yes_no(const char *title, const char *prompt, bool the_default);
+  // simple message box
+  virtual void message_box(const char *title, const char *message);
   // called at a regular interval, currently by the keyboard handler.
   virtual void periodic();
-  virtual int create_disk_image(const char *filename, int sectors, bx_bool overwrite);
+  virtual int create_disk_image(const char *filename, int sectors, bool overwrite);
   virtual void refresh_ci();
   virtual void refresh_vga() {
     if (init_done) {
@@ -155,8 +157,8 @@ public:
   bx_param_c *get_first_hd() {
     return get_first_atadevice(BX_ATA_DEVICE_DISK);
   }
-  virtual bx_bool is_pci_device(const char *name);
-  virtual bx_bool is_agp_device(const char *name);
+  virtual bool is_pci_device(const char *name);
+  virtual bool is_agp_device(const char *name);
 #if BX_DEBUGGER
   virtual void debug_break();
   virtual void debug_interpret_cmd(char *cmd);
@@ -173,21 +175,21 @@ public:
   virtual void unregister_runtime_config_handler(int id);
   virtual void update_runtime_options();
   virtual void set_sim_thread_func(is_sim_thread_func_t func) {}
-  virtual bx_bool is_sim_thread();
-  virtual void set_debug_gui(bx_bool val) { bx_debug_gui = val; }
-  virtual bx_bool has_debug_gui() const { return bx_debug_gui; }
-  virtual bx_bool is_wx_selected() const { return wxsel; }
+  virtual bool is_sim_thread();
+  virtual void set_debug_gui(bool val) { bx_debug_gui = val; }
+  virtual bool has_debug_gui() const { return bx_debug_gui; }
+  virtual bool is_wx_selected() const { return wxsel; }
   // provide interface to bx_gui->set_display_mode() method for config
   // interfaces to use.
   virtual void set_display_mode(disp_mode_t newmode) {
     if (bx_gui != NULL)
       bx_gui->set_display_mode(newmode);
   }
-  virtual bx_bool test_for_text_console();
+  virtual bool test_for_text_console();
   // add-on config option support
-  virtual bx_bool register_addon_option(const char *keyword, addon_option_parser_t parser, addon_option_save_t save_func);
-  virtual bx_bool unregister_addon_option(const char *keyword);
-  virtual bx_bool is_addon_option(const char *keyword);
+  virtual bool register_addon_option(const char *keyword, addon_option_parser_t parser, addon_option_save_t save_func);
+  virtual bool unregister_addon_option(const char *keyword);
+  virtual bool is_addon_option(const char *keyword);
   virtual Bit32s  parse_addon_option(const char *context, int num_params, char *params []);
   virtual Bit32s  save_addon_options(FILE *fp);
 
@@ -201,31 +203,38 @@ public:
   // save/restore support
   virtual void init_save_restore();
   virtual void cleanup_save_restore();
-  virtual bx_bool save_state(const char *checkpoint_path);
-  virtual bx_bool restore_config();
-  virtual bx_bool restore_logopts();
-  virtual bx_bool restore_hardware();
+  virtual bool save_state(const char *checkpoint_path);
+  virtual bool restore_config();
+  virtual bool restore_logopts();
+  virtual bool restore_hardware();
   virtual bx_list_c *get_bochs_root() {
     return (bx_list_c*)get_param("bochs", NULL);
   }
-  virtual bx_bool restore_bochs_param(bx_list_c *root, const char *sr_path, const char *restore_name);
+  virtual bool restore_bochs_param(bx_list_c *root, const char *sr_path, const char *restore_name);
   // special config parameter and options functions for plugins
-  virtual bx_bool opt_plugin_ctrl(const char *plugname, bx_bool load);
+  virtual bool opt_plugin_ctrl(const char *plugname, bool load);
+#if BX_NETWORKING
   virtual void init_std_nic_options(const char *name, bx_list_c *menu);
+#endif
+#if BX_SUPPORT_PCIUSB
   virtual void init_usb_options(const char *usb_name, const char *pname, int maxports);
+#endif
   virtual int  parse_param_from_list(const char *context, const char *param, bx_list_c *base);
   virtual int  parse_nic_params(const char *context, const char *param, bx_list_c *base);
-  virtual int  parse_usb_port_params(const char *context, bx_bool devopt,
-                                     const char *param, int maxports, bx_list_c *base);
-  virtual int  write_param_list(FILE *fp, bx_list_c *base, const char *optname, bx_bool multiline);
+  virtual int  parse_usb_port_params(const char *context, const char *param,
+                                     int maxports, bx_list_c *base);
+  virtual int  split_option_list(const char *msg, const char *rawopt, char **argv, int max_argv);
+  virtual int  write_param_list(FILE *fp, bx_list_c *base, const char *optname, bool multiline);
+#if BX_SUPPORT_PCIUSB
   virtual int  write_usb_options(FILE *fp, int maxports, bx_list_c *base);
+#endif
 #if BX_USE_GUI_CONSOLE
   virtual int  bx_printf(const char *fmt, ...);
   virtual char* bx_gets(char *s, int size, FILE *stream);
 #endif
 
 private:
-  bx_bool save_sr_param(FILE *fp, bx_param_c *node, const char *sr_path, int level);
+  bool save_sr_param(FILE *fp, bx_param_c *node, const char *sr_path, int level);
 };
 
 // recursive function to find parameters from the path
@@ -364,6 +373,21 @@ bx_real_sim_c::bx_real_sim_c()
   addon_options = NULL;
 }
 
+int bx_real_sim_c::set_init_done(bool n)
+{
+#if BX_USE_TEXTCONFIG
+  if (n) {
+    if (bx_gui->has_gui_console()) {
+      if (strcmp(registered_ci_name, "textconfig") != 0) {
+        PLUG_load_plugin(textconfig, PLUGTYPE_CORE);
+      }
+    }
+  }
+#endif
+  init_done = n;
+  return 0;
+}
+
 void bx_real_sim_c::reset_all_param()
 {
   bx_reset_options();
@@ -518,52 +542,6 @@ int floppy_type_n_sectors[] = { -1, 80*2*15, 80*2*18, 80*2*36, 80*2*9, 40*2*9, 4
 const char *media_status_names[] = { "ejected", "inserted", NULL };
 const char *bochs_bootdisk_names[] = { "none", "floppy", "disk","cdrom", "network", NULL };
 
-const char *sound_driver_names[] = {
-  "dummy",
-#if BX_HAVE_SOUND_ALSA
-  "alsa",
-#endif
-#if BX_HAVE_SOUND_OSS
-  "oss",
-#endif
-#if BX_HAVE_SOUND_OSX
-  "osx",
-#endif
-#if BX_HAVE_SOUND_SDL
-  "sdl",
-#endif
-#if BX_HAVE_SOUND_WIN
-  "win",
-#endif
-  "file",
-  NULL
-};
-
-const char *hdimage_mode_names[] = {
-  "flat",
-  "concat",
-  "external",
-  "dll",
-  "sparse",
-  "vmware3",
-  "vmware4",
-  "undoable",
-  "growing",
-  "volatile",
-  "vvfat",
-  "vpc",
-  "vbox",
-  NULL
-};
-
-int bx_real_sim_c::hdimage_get_mode(const char *mode)
-{
-  for (int i = 0; i <= BX_HDIMAGE_MODE_LAST; i++) {
-    if (!strcmp(mode, hdimage_mode_names[i])) return i;
-  }
-  return -1;
-}
-
 void bx_real_sim_c::set_notify_callback(bxevent_handler func, void *arg)
 {
   bxevent_callback = func;
@@ -656,7 +634,7 @@ int bx_real_sim_c::ask_filename(const char *filename, int maxlen, const char *pr
   return event.retcode;
 }
 
-int bx_real_sim_c::ask_yes_no(const char *title, const char *prompt, bx_bool the_default)
+int bx_real_sim_c::ask_yes_no(const char *title, const char *prompt, bool the_default)
 {
   BxEvent event;
   char format[512];
@@ -669,12 +647,21 @@ int bx_real_sim_c::ask_yes_no(const char *title, const char *prompt, bx_bool the
   sim_to_ci_event(&event);
   if (event.retcode >= 0) {
     return param.get();
-  }
-  else {
+  } else {
     return event.retcode;
   }
 }
 
+void bx_real_sim_c::message_box(const char *title, const char *message)
+{
+  BxEvent event;
+
+  event.type = BX_SYNC_EVT_MSG_BOX;
+  event.u.logmsg.prefix = title;
+  event.u.logmsg.msg = message;
+  sim_to_ci_event(&event);
+}
+
 void bx_real_sim_c::periodic()
 {
   // give the GUI a chance to do periodic things on the bochs thread. in
@@ -708,7 +695,7 @@ void bx_real_sim_c::periodic()
 // write, e.g. disk full.
 //
 // wxWidgets: This may be called from the gui thread.
-int bx_real_sim_c::create_disk_image(const char *filename, int sectors, bx_bool overwrite)
+int bx_real_sim_c::create_disk_image(const char *filename, int sectors, bool overwrite)
 {
   FILE *fp;
   if (!overwrite) {
@@ -786,12 +773,12 @@ bx_param_c *bx_real_sim_c::get_first_atadevice(Bit32u search_type)
   return NULL;
 }
 
-bx_bool bx_real_sim_c::is_pci_device(const char *name)
+bool bx_real_sim_c::is_pci_device(const char *name)
 {
 #if BX_SUPPORT_PCI
   unsigned i, max_pci_slots = BX_N_PCI_SLOTS;
   char devname[80];
-  char *device;
+  const char *device;
 
   if (SIM->get_param_bool(BXPN_PCI_ENABLED)->get()) {
     if (SIM->get_param_enum(BXPN_PCI_CHIPSET)->get() == BX_PCI_CHIPSET_I440BX) {
@@ -799,8 +786,8 @@ bx_bool bx_real_sim_c::is_pci_device(const char *name)
     }
     for (i = 0; i < max_pci_slots; i++) {
       sprintf(devname, "pci.slot.%d", i+1);
-      device = SIM->get_param_string(devname)->getptr();
-      if ((strlen(device) > 0) && (!strcmp(name, device))) {
+      device = SIM->get_param_enum(devname)->get_selected();
+      if (!strcmp(name, device)) {
         return 1;
       }
     }
@@ -809,13 +796,12 @@ bx_bool bx_real_sim_c::is_pci_device(const char *name)
   return 0;
 }
 
-bx_bool bx_real_sim_c::is_agp_device(const char *name)
+bool bx_real_sim_c::is_agp_device(const char *name)
 {
 #if BX_SUPPORT_PCI
-  if (get_param_bool(BXPN_PCI_ENABLED)->get() &&
-      (SIM->get_param_enum(BXPN_PCI_CHIPSET)->get() == BX_PCI_CHIPSET_I440BX)) {
-    const char *device = SIM->get_param_string("pci.slot.5")->getptr();
-    if ((strlen(device) > 0) && (!strcmp(name, device))) {
+  if (get_param_bool(BXPN_PCI_ENABLED)->get() && DEV_agp_present()) {
+    const char *device = SIM->get_param_enum("pci.slot.5")->get_selected();
+    if (!strcmp(name, device)) {
       return 1;
     }
   }
@@ -880,17 +866,11 @@ void bx_real_sim_c::register_configuration_interface(
 
 int bx_real_sim_c::configuration_interface(const char *ignore, ci_command_t command)
 {
-  bx_param_enum_c *ci_param = SIM->get_param_enum(BXPN_SEL_CONFIG_INTERFACE);
-  const char *name = ci_param->get_selected();
   if (!ci_callback) {
     BX_PANIC(("no configuration interface was loaded"));
     return -1;
   }
-  if (strcmp(name, registered_ci_name) != 0) {
-    BX_PANIC(("siminterface does not support loading one configuration interface and then calling another"));
-    return -1;
-  }
-  if (!strcmp(name, "wx"))
+  if (!strcmp(registered_ci_name, "wx"))
     wxsel = 1;
   else
     wxsel = 0;
@@ -960,7 +940,7 @@ void bx_real_sim_c::update_runtime_options()
   bx_virt_timer.set_realtime_delay();
 }
 
-bx_bool bx_real_sim_c::is_sim_thread()
+bool bx_real_sim_c::is_sim_thread()
 {
   if (is_sim_thread_func == NULL) return 1;
   return (*is_sim_thread_func)();
@@ -970,7 +950,7 @@ bx_bool bx_real_sim_c::is_sim_thread()
 // started from the "Start Menu" or by double clicking on it on a Mac,
 // there may be nothing attached to stdin/stdout/stderr.  This function
 // tests if stdin/stdout/stderr are usable and returns 0 if not.
-bx_bool bx_real_sim_c::test_for_text_console()
+bool bx_real_sim_c::test_for_text_console()
 {
 #if BX_WITH_CARBON
   // In a Carbon application, you have a text console if you run the app from
@@ -981,7 +961,7 @@ bx_bool bx_real_sim_c::test_for_text_console()
   return 1;
 }
 
-bx_bool bx_real_sim_c::is_addon_option(const char *keyword)
+bool bx_real_sim_c::is_addon_option(const char *keyword)
 {
   addon_option_t *addon_option;
 
@@ -991,7 +971,7 @@ bx_bool bx_real_sim_c::is_addon_option(const char *keyword)
   return 0;
 }
 
-bx_bool bx_real_sim_c::register_addon_option(const char *keyword, addon_option_parser_t parser,
+bool bx_real_sim_c::register_addon_option(const char *keyword, addon_option_parser_t parser,
                                              addon_option_save_t save_func)
 {
   addon_option_t *addon_option = new addon_option_t;
@@ -1017,7 +997,7 @@ bx_bool bx_real_sim_c::register_addon_option(const char *keyword, addon_option_p
   return 1;
 }
 
-bx_bool bx_real_sim_c::unregister_addon_option(const char *keyword)
+bool bx_real_sim_c::unregister_addon_option(const char *keyword)
 {
   addon_option_t *addon_option, *prev = NULL;
 
@@ -1091,7 +1071,7 @@ void bx_real_sim_c::cleanup_save_restore()
   }
 }
 
-bx_bool bx_real_sim_c::save_state(const char *checkpoint_path)
+bool bx_real_sim_c::save_state(const char *checkpoint_path)
 {
   char sr_file[BX_PATHNAME_LEN];
   char devname[20];
@@ -1136,7 +1116,7 @@ bx_bool bx_real_sim_c::save_state(const char *checkpoint_path)
   return 1;
 }
 
-bx_bool bx_real_sim_c::restore_config()
+bool bx_real_sim_c::restore_config()
 {
   char config[BX_PATHNAME_LEN];
   sprintf(config, "%s/config", get_param_string(BXPN_RESTORE_PATH)->getptr());
@@ -1144,7 +1124,7 @@ bx_bool bx_real_sim_c::restore_config()
   return (read_rc(config) >= 0);
 }
 
-bx_bool bx_real_sim_c::restore_logopts()
+bool bx_real_sim_c::restore_logopts()
 {
   char logopts[BX_PATHNAME_LEN];
   char line[512], string[512], devname[20];
@@ -1216,7 +1196,7 @@ static int bx_restore_getline(FILE *fp, char *line, int maxlen)
   return (ret != NULL) ? len : 0;
 }
 
-bx_bool bx_real_sim_c::restore_bochs_param(bx_list_c *root, const char *sr_path, const char *restore_name)
+bool bx_real_sim_c::restore_bochs_param(bx_list_c *root, const char *sr_path, const char *restore_name)
 {
   char devstate[BX_PATHNAME_LEN], devdata[BX_PATHNAME_LEN];
   char line[512], buf[512];
@@ -1305,14 +1285,18 @@ bx_bool bx_real_sim_c::restore_bochs_param(bx_list_c *root, const char *sr_path,
                   if (fp2 != NULL) {
                     FILE **fpp = ((bx_shadow_filedata_c*)param)->get_fpp();
                     // If the temporary backing store file wasn't created, do it now.
-                    if (*fpp == NULL)
-                      *fpp = tmpfile();
+                    if (*fpp == NULL) {
+                      *fpp = tmpfile64();
+                    } else {
+                      fseeko64(*fpp, 0, SEEK_SET);
+                    }
                     if (*fpp != NULL) {
+                      char *buffer = new char[4096];
                       while (!feof(fp2)) {
-                        char buffer[64];
-                        size_t chars = fread(buffer, 1, sizeof(buffer), fp2);
+                        size_t chars = fread(buffer, 1, 4096, fp2);
                         fwrite(buffer, 1, chars, *fpp);
                       }
+                      delete [] buffer;
                       fflush(*fpp);
                     }
                     ((bx_shadow_filedata_c*)param)->restore(fp2);
@@ -1341,7 +1325,7 @@ bx_bool bx_real_sim_c::restore_bochs_param(bx_list_c *root, const char *sr_path,
   return 1;
 }
 
-bx_bool bx_real_sim_c::restore_hardware()
+bool bx_real_sim_c::restore_hardware()
 {
   bx_list_c *sr_list = get_bochs_root();
   int ndev = sr_list->get_size();
@@ -1352,10 +1336,10 @@ bx_bool bx_real_sim_c::restore_hardware()
   return 1;
 }
 
-bx_bool bx_real_sim_c::save_sr_param(FILE *fp, bx_param_c *node, const char *sr_path, int level)
+bool bx_real_sim_c::save_sr_param(FILE *fp, bx_param_c *node, const char *sr_path, int level)
 {
   int i, j;
-  char pname[BX_PATHNAME_LEN], tmpstr[BX_PATHNAME_LEN];
+  char pname[BX_PATHNAME_LEN], tmpstr[BX_PATHNAME_LEN+1];
   FILE *fp2;
 
   for (i=0; i<level; i++)
@@ -1425,12 +1409,13 @@ bx_bool bx_real_sim_c::save_sr_param(FILE *fp, bx_param_c *node, const char *sr_
         FILE **fpp = ((bx_shadow_filedata_c*)node)->get_fpp();
         // If the backing store hasn't been created, just save an empty 0 byte placeholder file.
         if (*fpp != NULL) {
+          char *buffer = new char[4096];
+          fseeko64(*fpp, 0, SEEK_SET);
           while (!feof(*fpp)) {
-            char buffer[64];
-            size_t chars = fread (buffer, 1, sizeof(buffer), *fpp);
+            size_t chars = fread (buffer, 1, 4096, *fpp);
             fwrite(buffer, 1, chars, fp2);
           }
-          fflush(*fpp);
+          delete [] buffer;
         }
         ((bx_shadow_filedata_c*)node)->save(fp2);
         fclose(fp2);
@@ -1456,7 +1441,7 @@ bx_bool bx_real_sim_c::save_sr_param(FILE *fp, bx_param_c *node, const char *sr_
   return 1;
 }
 
-bx_bool bx_real_sim_c::opt_plugin_ctrl(const char *plugname, bx_bool load)
+bool bx_real_sim_c::opt_plugin_ctrl(const char *plugname, bool load)
 {
   bx_list_c *plugin_ctrl = (bx_list_c*)SIM->get_param(BXPN_PLUGIN_CTRL);
   if (!strcmp(plugname, "*")) {
@@ -1464,46 +1449,53 @@ bx_bool bx_real_sim_c::opt_plugin_ctrl(const char *plugname, bx_bool load)
     int i = 0;
     while (i < plugin_ctrl->get_size()) {
       bx_param_bool_c *plugin = (bx_param_bool_c*)plugin_ctrl->get(i);
-      if (load == (bx_bool)plugin->get()) {
+      if (load == (bool)plugin->get()) {
         opt_plugin_ctrl(plugin->get_name(), load);
-        if (load) i++;
-      } else {
-        i++;
       }
+      i++;
     }
     return 1;
   }
+  if (plugin_ctrl->get_by_name(plugname) == NULL) {
+    BX_PANIC(("Plugin '%s' not found", plugname));
+    return 0;
+  }
   if (load != PLUG_device_present(plugname)) {
     if (load) {
       if (PLUG_load_opt_plugin(plugname)) {
-        if (plugin_ctrl->get_by_name(plugname) == NULL) {
-          new bx_param_bool_c(plugin_ctrl, plugname, "", "", 1);
-        }
+        SIM->get_param_bool(plugname, plugin_ctrl)->set(1);
         return 1;
       } else {
-        BX_PANIC(("optional plugin '%s' not found", plugname));
+        // plugin load code panics in this case
         return 0;
       }
     } else {
-      PLUG_unload_opt_plugin(plugname);
-      plugin_ctrl->remove(plugname);
-      return 1;
+      if (PLUG_unload_opt_plugin(plugname)) {
+        SIM->get_param_bool(plugname, plugin_ctrl)->set(0);
+        return 1;
+      } else {
+        // plugin load code panics in this case
+        return 0;
+      }
     }
-  } else if (!load && !PLUG_device_present(plugname)) {
-    plugin_ctrl->remove(plugname);
   }
   return 0;
 }
 
+#if BX_NETWORKING
 void bx_real_sim_c::init_std_nic_options(const char *name, bx_list_c *menu)
 {
   bx_init_std_nic_options(name, menu);
 }
+#endif
 
+#if BX_SUPPORT_PCIUSB
 void bx_real_sim_c::init_usb_options(const char *usb_name, const char *pname, int maxports)
 {
   bx_init_usb_options(usb_name, pname, maxports);
 }
+#endif
+
 
 int bx_real_sim_c::parse_param_from_list(const char *context, const char *param, bx_list_c *base)
 {
@@ -1515,21 +1507,29 @@ int bx_real_sim_c::parse_nic_params(const char *context, const char *param, bx_l
   return bx_parse_nic_params(context, param, base);
 }
 
-int bx_real_sim_c::parse_usb_port_params(const char *context, bx_bool devopt,
-                                     const char *param, int maxports, bx_list_c *base)
+int bx_real_sim_c::parse_usb_port_params(const char *context, const char *param,
+                                         int maxports, bx_list_c *base)
 {
-  return bx_parse_usb_port_params(context, devopt, param, maxports, base);
+  return bx_parse_usb_port_params(context, param, maxports, base);
 }
 
-int bx_real_sim_c::write_param_list(FILE *fp, bx_list_c *base, const char *optname, bx_bool multiline)
+int bx_real_sim_c::split_option_list(const char *msg, const char *rawopt,
+                                     char **argv, int max_argv)
+{
+  return bx_split_option_list(msg, rawopt, argv, max_argv);
+}
+
+int bx_real_sim_c::write_param_list(FILE *fp, bx_list_c *base, const char *optname, bool multiline)
 {
   return bx_write_param_list(fp, base, optname, multiline);
 }
 
+#if BX_SUPPORT_PCIUSB
 int bx_real_sim_c::write_usb_options(FILE *fp, int maxports, bx_list_c *base)
 {
   return bx_write_usb_options(fp, maxports, base);
 }
+#endif
 
 #if BX_USE_GUI_CONSOLE
 int bx_real_sim_c::bx_printf(const char *fmt, ...)
