@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_xhci.h 13651 2019-12-09 15:14:23Z sshwarts $
+// $Id: usb_xhci.h 14158 2021-02-20 19:58:39Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2010-2017  Benjamin D Lunt (fys [at] fysnet [dot] net)
-//                2011-2017  The Bochs Project
+//                2011-2021  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -161,11 +161,11 @@
 // our saved ring members
 struct RING_MEMBERS {
   struct {
-    Bit64u  dq_pointer;
-    bx_bool rcs;
+    Bit64u dq_pointer;
+    bool   rcs;
   } command_ring;
   struct {
-    bx_bool  rcs;
+    bool     rcs;
     unsigned trb_count;
     unsigned count;
     Bit64u   cur_trb;
@@ -179,8 +179,8 @@ struct RING_MEMBERS {
 
 struct SLOT_CONTEXT {
   unsigned entries;
-  bx_bool  hub;
-  bx_bool  mtt;
+  bool     hub;
+  bool     mtt;
   unsigned speed;
   Bit32u   route_string;
   unsigned num_ports;
@@ -196,32 +196,32 @@ struct SLOT_CONTEXT {
 
 struct EP_CONTEXT {
   unsigned interval;
-  bx_bool  lsa;
+  bool     lsa;
   unsigned max_pstreams;
   unsigned mult;
   unsigned ep_state;
   unsigned max_packet_size;
   unsigned max_burst_size;
-  bx_bool  hid;
+  bool     hid;
   unsigned ep_type;
   unsigned cerr;
   Bit64u   tr_dequeue_pointer;
-  bx_bool  dcs;
+  bool     dcs;
   unsigned max_esit_payload;
   unsigned average_trb_len;
 };
 
 struct HC_SLOT_CONTEXT {
-  bx_bool enabled;
-  bx_bool sent_address;  // have we sent a SET_ADDRESS command yet?
+  bool enabled;
+  bool sent_address;  // have we sent a SET_ADDRESS command yet?
   struct SLOT_CONTEXT slot_context;
   struct {
     struct EP_CONTEXT   ep_context;
     // our internal registers follow
     Bit32u  edtla;
     Bit64u  enqueue_pointer;
-    bx_bool rcs;
-    bx_bool retry;
+    bool    rcs;
+    bool    retry;
     int     retry_counter;
   } ep_context[32];  // first one is ignored by controller.
 };
@@ -335,119 +335,119 @@ typedef struct {
 
   struct XHCI_OP_REGS {
     struct {
-      Bit32u  RsvdP1;            // 18/20 bit reserved and preserved   = 0x000000       RW
+      Bit32u RsvdP1;         // 18/20 bit reserved and preserved   = 0x000000       RW
 #if ((VERSION_MAJOR == 1) && (VERSION_MINOR >= 0x10))
-      bx_bool cme;               //  1 bit Max Exit Latecy to Large    = 0b             RW
-      bx_bool spe;               //  1 bit Generate Short Packet Comp  = 0b             RW
+      bool   cme;            //  1 bit Max Exit Latecy to Large    = 0b             RW
+      bool   spe;            //  1 bit Generate Short Packet Comp  = 0b             RW
 #endif
-      bx_bool eu3s;              //  1 bit Enable U3 MFINDEX Stop      = 0b             RW
-      bx_bool ewe;               //  1 bit Enable Wrap Event           = 0b             RW
-      bx_bool crs;               //  1 bit Controller Restore State    = 0b             RW
-      bx_bool css;               //  1 bit Controller Save State       = 0b             RW
-      bx_bool lhcrst;            //  1 bit Light HC Reset              = 0b             RW or RO (HCCPARAMS:LHRC)
-      Bit8u   RsvdP0;            //  1 bit reserved and preserved      = 000b           RW
-      bx_bool hsee;              //  1 bit Host System Error Enable    = 0b             RW
-      bx_bool inte;              //  1 bit Interrupter Enable          = 0b             RW
-      bx_bool hcrst;             //  1 bit HC Reset                    = 0b             RW
-      bx_bool rs;                //  1 bit Run Stop                    = 0b             RW
-    } HcCommand;                 //                                    = 0x00000000
+      bool   eu3s;           //  1 bit Enable U3 MFINDEX Stop      = 0b             RW
+      bool   ewe;            //  1 bit Enable Wrap Event           = 0b             RW
+      bool   crs;            //  1 bit Controller Restore State    = 0b             RW
+      bool   css;            //  1 bit Controller Save State       = 0b             RW
+      bool   lhcrst;         //  1 bit Light HC Reset              = 0b             RW or RO (HCCPARAMS:LHRC)
+      Bit8u  RsvdP0;         //  1 bit reserved and preserved      = 000b           RW
+      bool   hsee;           //  1 bit Host System Error Enable    = 0b             RW
+      bool   inte;           //  1 bit Interrupter Enable          = 0b             RW
+      bool   hcrst;          //  1 bit HC Reset                    = 0b             RW
+      bool   rs;             //  1 bit Run Stop                    = 0b             RW
+    } HcCommand;             //                                    = 0x00000000
     struct {
-      Bit32u  RsvdZ1;            // 19 bit reserved and zero'd         = 0x000000       RW
-      bx_bool hce;               //  1 bit Host Controller Error       = 0b             RO
-      bx_bool cnr;               //  1 bit Controller Not Ready        = 0b             R0
-      bx_bool sre;               //  1 bit Save/Restore Error          = 0b             RW1C
-      bx_bool rss;               //  1 bit Restore State Status        = 0b             RO
-      bx_bool sss;               //  1 bit Save State Status           = 0b             RO
-      Bit8u   RsvdZ0;            //  3 bit reserved and zero'd         = 0x0            RW
-      bx_bool pcd;               //  1 bit Port Change Detect          = 0b             RW1C
-      bx_bool eint;              //  1 bit Event Interrupt             = 0b             RW1C
-      bx_bool hse;               //  1 bit Host System Error           = 0b             RW1C
-      bx_bool RsvdZ2;            //  1 bit reserved and zero'd         = 0b             RW
-      bx_bool hch;               //  1 bit HCHalted                    = 1b             RO
-    } HcStatus;                  //                                    = 0x00000001
+      Bit32u RsvdZ1;         // 19 bit reserved and zero'd         = 0x000000       RW
+      bool   hce;            //  1 bit Host Controller Error       = 0b             RO
+      bool   cnr;            //  1 bit Controller Not Ready        = 0b             R0
+      bool   sre;            //  1 bit Save/Restore Error          = 0b             RW1C
+      bool   rss;            //  1 bit Restore State Status        = 0b             RO
+      bool   sss;            //  1 bit Save State Status           = 0b             RO
+      Bit8u  RsvdZ0;         //  3 bit reserved and zero'd         = 0x0            RW
+      bool   pcd;            //  1 bit Port Change Detect          = 0b             RW1C
+      bool   eint;           //  1 bit Event Interrupt             = 0b             RW1C
+      bool   hse;            //  1 bit Host System Error           = 0b             RW1C
+      bool   RsvdZ2;         //  1 bit reserved and zero'd         = 0b             RW
+      bool   hch;            //  1 bit HCHalted                    = 1b             RO
+    } HcStatus;              //                                    = 0x00000001
     struct {
-      Bit16u  Rsvd;              // 16 bit reserved                    = 0x0000         RO
-      Bit16u  pagesize;          // 16 bit reserved                    = 0x0001         RO
-    } HcPageSize;                //                                    = 0x00000001
+      Bit16u  Rsvd;          // 16 bit reserved                    = 0x0000         RO
+      Bit16u  pagesize;      // 16 bit reserved                    = 0x0001         RO
+    } HcPageSize;            //                                    = 0x00000001
     struct {
-      Bit16u  RsvdP;             // 16 bit reserved and presserved     = 0x0000         RW
-      bx_bool n15;               //  1 bit N15                         = 0              RW
-      bx_bool n14;               //  1 bit N14                         = 0              RW
-      bx_bool n13;               //  1 bit N13                         = 0              RW
-      bx_bool n12;               //  1 bit N12                         = 0              RW
-      bx_bool n11;               //  1 bit N11                         = 0              RW
-      bx_bool n10;               //  1 bit N10                         = 0              RW
-      bx_bool n9;                //  1 bit N9                          = 0              RW
-      bx_bool n8;                //  1 bit N8                          = 0              RW
-      bx_bool n7;                //  1 bit N7                          = 0              RW
-      bx_bool n6;                //  1 bit N6                          = 0              RW
-      bx_bool n5;                //  1 bit N5                          = 0              RW
-      bx_bool n4;                //  1 bit N4                          = 0              RW
-      bx_bool n3;                //  1 bit N3                          = 0              RW
-      bx_bool n2;                //  1 bit N2                          = 0              RW
-      bx_bool n1;                //  1 bit N1                          = 0              RW
-      bx_bool n0;                //  1 bit N0                          = 0              RW
-    } HcNotification;            //                                    = 0x00000000
+      Bit16u  RsvdP;         // 16 bit reserved and presserved     = 0x0000         RW
+      bool n15;              //  1 bit N15                         = 0              RW
+      bool n14;              //  1 bit N14                         = 0              RW
+      bool n13;              //  1 bit N13                         = 0              RW
+      bool n12;              //  1 bit N12                         = 0              RW
+      bool n11;              //  1 bit N11                         = 0              RW
+      bool n10;              //  1 bit N10                         = 0              RW
+      bool n9;               //  1 bit N9                          = 0              RW
+      bool n8;               //  1 bit N8                          = 0              RW
+      bool n7;               //  1 bit N7                          = 0              RW
+      bool n6;               //  1 bit N6                          = 0              RW
+      bool n5;               //  1 bit N5                          = 0              RW
+      bool n4;               //  1 bit N4                          = 0              RW
+      bool n3;               //  1 bit N3                          = 0              RW
+      bool n2;               //  1 bit N2                          = 0              RW
+      bool n1;               //  1 bit N1                          = 0              RW
+      bool n0;               //  1 bit N0                          = 0              RW
+    } HcNotification;        //                                    = 0x00000000
     struct {
-      Bit64u  crc;               // 64 bit hi order address            = 0x00000000     RW
-      Bit8u   RsvdP;             //  2 bit reserved and preserved      = 00b            RW
-      bx_bool crr;               //  1 bit Command Ring Running        = 0              RO
-      bx_bool ca;                //  1 bit Command Abort               = 0              RW1S
-      bx_bool cs;                //  1 bit Command Stop                = 0              RW1S
-      bx_bool rcs;               //  1 bit Ring Cycle State            = 0              RW
+      Bit64u crc;            // 64 bit hi order address            = 0x00000000     RW
+      Bit8u  RsvdP;          //  2 bit reserved and preserved      = 00b            RW
+      bool   crr;            //  1 bit Command Ring Running        = 0              RO
+      bool   ca;             //  1 bit Command Abort               = 0              RW1S
+      bool   cs;             //  1 bit Command Stop                = 0              RW1S
+      bool   rcs;            //  1 bit Ring Cycle State            = 0              RW
     } HcCrcr;
     struct {
-      Bit64u  dcbaap;            // 64 bit hi order address            = 0x00000000     RW
-      Bit8u   RsvdZ;             //  6 bit reserved and zero'd         = 000000b        RW
+      Bit64u dcbaap;         // 64 bit hi order address            = 0x00000000     RW
+      Bit8u  RsvdZ;          //  6 bit reserved and zero'd         = 000000b        RW
     } HcDCBAAP;
     struct {
-      Bit32u  RsvdP;             // 22/24 bit reserved and preserved   = 0x000000       RW
+      Bit32u RsvdP;          // 22/24 bit reserved and preserved   = 0x000000       RW
 #if ((VERSION_MAJOR == 1) && (VERSION_MINOR >= 0x10))
-      bx_bool u3e;               //  1 bit U3 Entry Enable             = 0              RW
-      bx_bool cie;               //  1 bit Config Info Enable          = 0              RW
+      bool   u3e;            //  1 bit U3 Entry Enable             = 0              RW
+      bool   cie;            //  1 bit Config Info Enable          = 0              RW
 #endif
-      Bit8u   MaxSlotsEn;        //  8 bit Max Device Slots Enabled    = 0x00           RW
+      Bit8u   MaxSlotsEn;    //  8 bit Max Device Slots Enabled    = 0x00           RW
     } HcConfig;
   } op_regs;
 
   struct {
     // our data
-    usb_device_c *device;   // device connected to this port
-    bx_bool is_usb3;        // set if usb3 port, cleared if usb2 port.
-    bx_bool has_been_reset; // set if the port has been reset aftet powered up.
+    usb_device_c *device; // device connected to this port
+    bool is_usb3;         // set if usb3 port, cleared if usb2 port.
+    bool has_been_reset;  // set if the port has been reset aftet powered up.
 
     struct {
-      bx_bool wpr;               //  1 bit Warm Port Reset             = 0b             RW or RsvdZ
-      bx_bool dr;                //  1 bit Device Removable            = 0b             RO
-      Bit8u   RsvdZ1;            //  2 bit Reserved and Zero'd         = 00b            RW
-      bx_bool woe;               //  1 bit Wake on Over Current Enable = 0b             RW
-      bx_bool wde;               //  1 bit Wake on Disconnect Enable   = 0b             RW
-      bx_bool wce;               //  1 bit Wake on Connect Enable      = 0b             RW
-      bx_bool cas;               //  1 bit Cold Attach Status          = 0b             RO
-      bx_bool cec;               //  1 bit Port Config Error Change    = 0b             RW1C or RsvdZ
-      bx_bool plc;               //  1 bit Port Link State Change      = 0b             RW1C
-      bx_bool prc;               //  1 bit Port Reset Change           = 0b             RW1C
-      bx_bool occ;               //  1 bit Over Current Change         = 0b             RW1C
-      bx_bool wrc;               //  1 bit Warm Port Reset Change      = 0b             RW1C or RsvdZ
-      bx_bool pec;               //  1 bit Port Enabled/Disabled Change= 0b             RW1C
-      bx_bool csc;               //  1 bit Connect Status Change       = 0b             RW1C
-      bx_bool lws;               //  1 bit Port Link State Write Strobe= 0b             RW
-      Bit8u   pic;               //  2 bit Port Indicator Control      = 00b            RW
-      Bit8u   speed;             //  4 bit Port Speed                  = 0000b          RO
-      bx_bool pp;                //  1 bit Port Power                  = 0b             RW
-      Bit8u   pls;               //  4 bit Port Link State             = 0x00           RW
-      bx_bool pr;                //  1 bit Port Reset                  = 0b             RW
-      bx_bool oca;               //  1 bit Over Current Active         = 0b             RO
-      bx_bool RsvdZ0;            //  1 bit Reserved and Zero'd         = 0b             RW
-      bx_bool ped;               //  1 bit Port Enabled/Disabled       = 0b             RW1C
-      bx_bool ccs;               //  1 bit Current Connect Status      = 0b             RO
+      bool  wpr;               //  1 bit Warm Port Reset             = 0b             RW or RsvdZ
+      bool  dr;                //  1 bit Device Removable            = 0b             RO
+      Bit8u RsvdZ1;            //  2 bit Reserved and Zero'd         = 00b            RW
+      bool  woe;               //  1 bit Wake on Over Current Enable = 0b             RW
+      bool  wde;               //  1 bit Wake on Disconnect Enable   = 0b             RW
+      bool  wce;               //  1 bit Wake on Connect Enable      = 0b             RW
+      bool  cas;               //  1 bit Cold Attach Status          = 0b             RO
+      bool  cec;               //  1 bit Port Config Error Change    = 0b             RW1C or RsvdZ
+      bool  plc;               //  1 bit Port Link State Change      = 0b             RW1C
+      bool  prc;               //  1 bit Port Reset Change           = 0b             RW1C
+      bool  occ;               //  1 bit Over Current Change         = 0b             RW1C
+      bool  wrc;               //  1 bit Warm Port Reset Change      = 0b             RW1C or RsvdZ
+      bool  pec;               //  1 bit Port Enabled/Disabled Change= 0b             RW1C
+      bool  csc;               //  1 bit Connect Status Change       = 0b             RW1C
+      bool  lws;               //  1 bit Port Link State Write Strobe= 0b             RW
+      Bit8u pic;               //  2 bit Port Indicator Control      = 00b            RW
+      Bit8u speed;             //  4 bit Port Speed                  = 0000b          RO
+      bool  pp;                //  1 bit Port Power                  = 0b             RW
+      Bit8u pls;               //  4 bit Port Link State             = 0x00           RW
+      bool  pr;                //  1 bit Port Reset                  = 0b             RW
+      bool  oca;               //  1 bit Over Current Active         = 0b             RO
+      bool  RsvdZ0;            //  1 bit Reserved and Zero'd         = 0b             RW
+      bool  ped;               //  1 bit Port Enabled/Disabled       = 0b             RW1C
+      bool  ccs;               //  1 bit Current Connect Status      = 0b             RO
     } portsc;
     union {
       // if usb3 port
       struct {
         struct {
           Bit16u  RsvdP;         // 15 bit Reserved and Preserved      = 0x0000         RW
-          bx_bool fla;           //  1 bit Force Link PM Accept        = 0x0000         RW
+          bool    fla;           //  1 bit Force Link PM Accept        = 0x0000         RW
           Bit8u   u2timeout;     //  8 bit U2 Timeout                  = 0x0000         RW
           Bit8u   u1timeout;     //  8 bit U1 Timeout                  = 0x0000         RW
         } portpmsc;
@@ -461,10 +461,10 @@ typedef struct {
         struct {
           Bit8u   tmode;         //  4 bit Test Mode                   = 0x0            RO
           Bit16u  RsvdP;         // 11 bit reserved and preseved       = 0x000          RW
-          bx_bool hle;           //  1 bit hardware LPM enable         = 0b             RW
+          bool    hle;           //  1 bit hardware LPM enable         = 0b             RW
           Bit8u   l1dslot;       //  8 bit L1 Device Slot              = 0x00           RW
           Bit8u   hird;          //  4 bit Host Initiated Resume Durat = 0x0            RW
-          bx_bool rwe;           //  1 bit Remote Wakeup Enable        = 0b             RW
+          bool    rwe;           //  1 bit Remote Wakeup Enable        = 0b             RW
           Bit8u   l1s;           //  3 bit L1 Status                   = 000b           RO
         } portpmsc;
         struct {
@@ -491,8 +491,8 @@ typedef struct {
     struct {
       struct {
         Bit32u  RsvdP;           // 30 bit reserved and preseved       = 0x00000000     RW
-        bx_bool ie;              //  1 bit Interrupt Enable            = 0b             RW
-        bx_bool ip;              //  1 bit Interrupt Pending           = 0b             RW1C
+        bool    ie;              //  1 bit Interrupt Enable            = 0b             RW
+        bool    ip;              //  1 bit Interrupt Pending           = 0b             RW1C
       } iman;
       struct {
         Bit16u  imodc;           // 16 bit Interrupter Mod Counter     = 0x0000         RW
@@ -509,7 +509,7 @@ typedef struct {
       } erstba;
       struct {
         Bit64u  eventadd;        // 64 bit Event Ring Addy hi          = 0x00000000     RW
-        bx_bool ehb;             //  1 bit Event Handler Busy          = 0b             RW1C
+        bool    ehb;             //  1 bit Event Handler Busy          = 0b             RW1C
         Bit8u   desi;            //  2 bit Dequeue ERST Seg Index      = 00b            RW
       } erdp;
     } interrupter[INTERRUPTERS];
@@ -545,9 +545,6 @@ public:
 
   void event_handler(int event, USBPacket *packet, int port);
 
-  static const char *usb_param_handler(bx_param_string_c *param, int set,
-                                       const char *oldval, const char *val, int maxlen);
-
 private:
   bx_usb_xhci_t hub;
   Bit8u         devfunc;
@@ -559,14 +556,14 @@ private:
   static void reset_hc();
   static void reset_port(int);
   static void reset_port_usb3(int, const int);
-  static bx_bool save_hc_state(void);
-  static bx_bool restore_hc_state(void);
+  static bool save_hc_state(void);
+  static bool restore_hc_state(void);
 
   static void update_irq(unsigned interrupter);
 
   static void init_device(Bit8u port, bx_list_c *portconf);
   static void remove_device(Bit8u port);
-  static void usb_set_connect_status(Bit8u port, int type, bx_bool connected);
+  static bool usb_set_connect_status(Bit8u port, bool connected);
 
   static int  broadcast_packet(USBPacket *p, const int port);
   static void xhci_timer_handler(void *);
@@ -575,7 +572,7 @@ private:
   static void process_transfer_ring(const int slot, const int ep);
   static void process_command_ring(void);
   static void write_event_TRB(const unsigned interrupter, const Bit64u parameter, const Bit32u status, 
-                              const Bit32u command, const bx_bool fire_int);
+                              const Bit32u command, const bool fire_int);
   static Bit32u NEC_verification(const Bit64u parameter);
   static void init_event_ring(const unsigned interrupter);
   static void read_TRB(bx_phy_address addr, struct TRB *trb);
@@ -588,23 +585,26 @@ private:
   static void copy_ep_from_buffer(struct EP_CONTEXT *ep_context, const Bit8u *buffer);
   static void copy_slot_to_buffer(Bit32u *buffer, const int slot);
   static void copy_ep_to_buffer(Bit32u *buffer, const int slot, const int ep);
-  static bx_bool validate_slot_context(const struct SLOT_CONTEXT *slot_context);
-  static bx_bool validate_ep_context(const struct EP_CONTEXT *ep_context, int speed, int ep_num);
+  static bool validate_slot_context(const struct SLOT_CONTEXT *slot_context);
+  static bool validate_ep_context(const struct EP_CONTEXT *ep_context, int speed, int ep_num);
   static int  create_unique_address(const int slot);
   static int  send_set_address(const int addr, const int port_num);
 
   static void dump_xhci_core(const int slots, const int eps);
 
 #if BX_USE_USB_XHCI_SMF
-  static bx_bool read_handler(bx_phy_address addr, unsigned len, void *data, void *param);
-  static bx_bool write_handler(bx_phy_address addr, unsigned len, void *data, void *param);
+  static bool read_handler(bx_phy_address addr, unsigned len, void *data, void *param);
+  static bool write_handler(bx_phy_address addr, unsigned len, void *data, void *param);
 #else
-  bx_bool read_handler(bx_phy_address addr, unsigned len, void *data, void *param);
-  bx_bool write_handler(bx_phy_address addr, unsigned len, void *data, void *param);
+  bool read_handler(bx_phy_address addr, unsigned len, void *data, void *param);
+  bool write_handler(bx_phy_address addr, unsigned len, void *data, void *param);
 #endif
 
   static void runtime_config_handler(void *);
   void runtime_config(void);
+
+  static Bit64s usb_param_handler(bx_param_c *param, bool set, Bit64s val);
+  static bool usb_param_enable_handler(bx_param_c *param, bool en);
 };
 
 #endif  // BX_IODEV_USB_XHCI_H
