@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: plugin.cc 13302 2017-10-10 18:06:16Z vruppert $
+// $Id: plugin.cc 14214 2021-04-03 18:26:03Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2017  The Bochs Project
+//  Copyright (C) 2002-2021  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -34,30 +34,33 @@
 #include "iodev/iodev.h"
 #include "plugin.h"
 
+#ifndef WIN32
+#include <dirent.h> /* opendir, readdir */
+#include <locale.h> /* setlocale */
+#endif
+
 #define LOG_THIS genlog->
 
-#define PLUGIN_INIT_FMT_STRING       "lib%s_LTX_plugin_init"
-#define PLUGIN_FINI_FMT_STRING       "lib%s_LTX_plugin_fini"
-#define GUI_PLUGIN_INIT_FMT_STRING   "lib%s_gui_plugin_init"
-#define GUI_PLUGIN_FINI_FMT_STRING   "lib%s_gui_plugin_fini"
-#define SOUND_PLUGIN_INIT_FMT_STRING "lib%s_sound_plugin_init"
-#define SOUND_PLUGIN_FINI_FMT_STRING "lib%s_sound_plugin_fini"
-#define NET_PLUGIN_INIT_FMT_STRING   "lib%s_net_plugin_init"
-#define NET_PLUGIN_FINI_FMT_STRING   "lib%s_net_plugin_fini"
-#define USB_PLUGIN_INIT_FMT_STRING   "lib%s_dev_plugin_init"
-#define USB_PLUGIN_FINI_FMT_STRING   "lib%s_dev_plugin_fini"
-#define PLUGIN_PATH                  ""
+#define PLUGIN_ENTRY_FMT_STRING       "lib%s_plugin_entry"
+#define GUI_PLUGIN_ENTRY_FMT_STRING   "lib%s_gui_plugin_entry"
+#define IMG_PLUGIN_ENTRY_FMT_STRING   "lib%s_img_plugin_entry"
+#define NET_PLUGIN_ENTRY_FMT_STRING   "libeth_%s_plugin_entry"
+#define SND_PLUGIN_ENTRY_FMT_STRING   "libsound%s_plugin_entry"
+
+#define PLUGIN_PATH                   ""
 
 #ifndef WIN32
 #define PLUGIN_FILENAME_FORMAT       "libbx_%s.so"
-#define SOUND_PLUGIN_FILENAME_FORMAT "libbx_sound%s.so"
+#define GUI_PLUGIN_FILENAME_FORMAT   "libbx_%s_gui.so"
+#define IMG_PLUGIN_FILENAME_FORMAT   "libbx_%s_img.so"
 #define NET_PLUGIN_FILENAME_FORMAT   "libbx_eth_%s.so"
-#define USB_PLUGIN_FILENAME_FORMAT   "libbx_%s.so"
+#define SND_PLUGIN_FILENAME_FORMAT   "libbx_sound%s.so"
 #else
 #define PLUGIN_FILENAME_FORMAT       "bx_%s.dll"
-#define SOUND_PLUGIN_FILENAME_FORMAT "bx_sound%s.dll"
+#define GUI_PLUGIN_FILENAME_FORMAT   "bx_%s_gui.dll"
+#define IMG_PLUGIN_FILENAME_FORMAT   "bx_%s_img.dll"
 #define NET_PLUGIN_FILENAME_FORMAT   "bx_eth_%s.dll"
-#define USB_PLUGIN_FILENAME_FORMAT   "bx_%s.dll"
+#define SND_PLUGIN_FILENAME_FORMAT "bx_sound%s.dll"
 #endif
 
 logfunctions *pluginlog;
@@ -95,10 +98,6 @@ void (*pluginHRQHackCallback)(void);
 unsigned pluginHRQ = 0;
 
 plugin_t *plugins = NULL;      /* Head of the linked list of plugins  */
-#if BX_PLUGINS
-static void plugin_init_one(plugin_t *plugin);
-#endif
-
 device_t *devices = NULL;      /* Head of the linked list of registered devices  */
 device_t *core_devices = NULL; /* Head of the linked list of registered core devices  */
 
@@ -111,41 +110,25 @@ plugin_t *current_plugin_context = NULL;
   static void
 builtinRegisterIRQ(unsigned irq, const char* name)
 {
-#if 0
-  pluginlog->panic("builtinRegisterIRQ called, no pic plugin loaded?");
-#else
   bx_devices.register_irq(irq, name);
-#endif
 }
 
   static void
 builtinUnregisterIRQ(unsigned irq, const char* name)
 {
-#if 0
-  pluginlog->panic("builtinUnregisterIRQ called, no pic plugin loaded?");
-#else
   bx_devices.unregister_irq(irq, name);
-#endif
 }
 
   static void
 builtinSetHRQ(unsigned val)
 {
-#if 0
-  pluginlog->panic("builtinSetHRQ called, no plugin loaded?");
-#else
   pluginHRQ = val;
-#endif
 }
 
   static void
 builtinSetHRQHackCallback(void (*callback)(void))
 {
-#if 0
-  pluginlog->panic("builtinSetHRQHackCallback called, no plugin loaded?");
-#else
   pluginHRQHackCallback = callback;
-#endif
 }
 
   static int
@@ -257,47 +240,248 @@ builtinRegisterDefaultIOWriteHandler(void *thisPtr, ioWriteHandler_t callback,
 }
 
 #if BX_PLUGINS
+/************************************************************************/
+/* Search for all available plugins                                           */
+/************************************************************************/
+
+void plugin_add_entry(char *pgn_name, Bit16u type, Bit8u flags)
+{
+  plugin_t *plugin, *temp;
+
+  if ((type & PLUGTYPE_GUI) > 0) {
+    if (!strncmp(pgn_name + strlen(pgn_name) - 4, "_gui", 4)) {
+      pgn_name[strlen(pgn_name) - 4] = 0;
+    }
+  } else if (type == PLUGTYPE_IMG) {
+    if (!strncmp(pgn_name + strlen(pgn_name) - 4, "_img", 4)) {
+      pgn_name[strlen(pgn_name) - 4] = 0;
+    }
+  } else if (type == PLUGTYPE_NET) {
+    if (!strncmp(pgn_name, "eth_", 4)) {
+      memmove(pgn_name, pgn_name + 4, strlen(pgn_name) - 3);
+    }
+  } else if (type == PLUGTYPE_SND) {
+    if (!strncmp(pgn_name, "sound", 5)) {
+      memmove(pgn_name, pgn_name + 5, strlen(pgn_name) - 4);
+    }
+  }
+  plugin = new plugin_t;
+  plugin->type = type;
+  plugin->flags = flags;
+  plugin->name = pgn_name;
+  plugin->loadtype = PLUGTYPE_NULL;
+  plugin->initialized = 0;
+  /* Insert plugin at the _end_ of the plugin linked list. */
+  plugin->next = NULL;
+  if (plugins == NULL) {
+    /* Empty list, this become the first entry. */
+    plugins = plugin;
+  } else {
+    /* Non-empty list.  Add to end. */
+    temp = plugins;
+
+    while (temp->next)
+      temp = temp->next;
+
+    temp->next = plugin;
+  }
+}
+
+void plugins_search(void)
+{
+  int nlen, flen1, flen2;
+  char *fmtptr, *ltdl_path_var, *pgn_name, *pgn_path, *ptr;
+  char fmtstr[32];
+#ifndef WIN32
+  const char *path_sep = ":";
+  DIR *dir;
+  struct dirent *dent;
+  lt_dlhandle handle;
+#else
+  const char *path_sep = ";";
+  WIN32_FIND_DATA finddata;
+  HANDLE hFind;
+  char filter[MAX_PATH];
+  char path[MAX_PATH];
+  HINSTANCE handle;
+#endif
+  char tmpname[BX_PATHNAME_LEN];
+  plugin_entry_t plugin_entry;
+  Bit16u type;
+  Bit8u flags;
+
+#ifndef WIN32
+  setlocale(LC_ALL, "en_US");
+#endif
+  ltdl_path_var = getenv("LTDL_LIBRARY_PATH");
+  sprintf(fmtstr, PLUGIN_FILENAME_FORMAT, "*");
+  fmtptr = strchr(fmtstr, '*');
+  flen1 = fmtptr - fmtstr;
+  flen2 = strlen(fmtstr) - flen1 - 1;
+  if (ltdl_path_var != NULL) {
+    pgn_path = new char[strlen(ltdl_path_var) + 1];
+    strcpy(pgn_path, ltdl_path_var);
+  } else {
+    pgn_path = new char[2];
+    strcpy(pgn_path, ".");
+  }
+  ptr = strtok(pgn_path, path_sep);
+  while (ptr != NULL) {
+#ifndef WIN32
+    dir = opendir(ptr);
+    if (dir != NULL) {
+      while ((dent=readdir(dir)) != NULL) {
+        nlen = strlen(dent->d_name);
+        if ((!strncmp(dent->d_name, fmtstr, flen1)) &&
+            (!strcmp(dent->d_name + nlen - flen2, fmtptr + 1))) {
+          pgn_name = new char[nlen - flen1 - flen2 + 1];
+          strncpy(pgn_name, dent->d_name + flen1, nlen - flen1 - flen2);
+          pgn_name[nlen - flen1 - flen2] = 0;
+          handle = lt_dlopen(dent->d_name);
+          if (handle) {
+            sprintf(tmpname, PLUGIN_ENTRY_FMT_STRING, pgn_name);
+            plugin_entry = (plugin_entry_t) lt_dlsym(handle, tmpname);
+            if (plugin_entry != NULL) {
+              type = (Bit16u) plugin_entry(NULL, PLUGTYPE_NULL, PLUGIN_PROBE);
+              flags = (Bit8u) plugin_entry(NULL, PLUGTYPE_NULL, PLUGIN_FLAGS);
+              plugin_add_entry(pgn_name, type, flags);
+            }
+            lt_dlclose(handle);
+          } else {
+            delete [] pgn_name;
+          }
+        }
+      }
+      closedir(dir);
+    }
+#else
+    sprintf(filter, "%s\\*.dll", ptr);
+    hFind = FindFirstFile(filter, &finddata);
+    if (hFind != INVALID_HANDLE_VALUE) {
+      do {
+        nlen = lstrlen(finddata.cFileName);
+        if ((!strncmp(finddata.cFileName, fmtstr, flen1)) &&
+            (!strcmp(finddata.cFileName + nlen - flen2, fmtptr + 1))) {
+          pgn_name = new char[nlen - flen1 - flen2 + 1];
+          strncpy(pgn_name, finddata.cFileName + flen1, nlen - flen1 - flen2);
+          pgn_name[nlen - flen1 - flen2] = 0;
+          sprintf(path, "%s\\%s", ptr, finddata.cFileName);
+          handle = LoadLibrary(path);
+          if (handle) {
+            sprintf(tmpname, PLUGIN_ENTRY_FMT_STRING, pgn_name);
+            plugin_entry = (plugin_entry_t) GetProcAddress(handle, tmpname);
+            if (plugin_entry != NULL) {
+              type = (Bit16u) plugin_entry(NULL, PLUGTYPE_NULL, PLUGIN_PROBE);
+              flags = (Bit8u) plugin_entry(NULL, PLUGTYPE_NULL, PLUGIN_FLAGS);
+              plugin_add_entry(pgn_name, type, flags);
+            }
+            FreeLibrary(handle);
+          }
+        }
+      } while (FindNextFile(hFind, &finddata));
+      FindClose(hFind);
+    }
+#endif
+    ptr = strtok(NULL, ":");
+  }
+  delete [] pgn_path;
+}
+
+Bit8u bx_get_plugins_count(Bit16u type)
+{
+  plugin_t *temp;
+  Bit8u count = 0;
+
+  if (plugins != NULL) {
+    temp = plugins;
+
+    while (temp != NULL) {
+      if ((type & temp->type) != 0)
+        count++;
+      temp = temp->next;
+    }
+  }
+  return count;
+}
+
+const char* bx_get_plugin_name(Bit16u type, Bit8u index)
+{
+  plugin_t *temp;
+  int count = 0;
+
+  if (plugins != NULL) {
+    temp = plugins;
+
+    while (temp != NULL) {
+      if ((type & temp->type) != 0) {
+        if (count == index)
+          return temp->name;
+        count++;
+      }
+      temp = temp->next;
+    }
+  }
+  return NULL;
+}
+
+Bit8u bx_get_plugin_flags(Bit16u type, Bit8u index)
+{
+  plugin_t *temp;
+  int count = 0;
+
+  if (plugins != NULL) {
+    temp = plugins;
+
+    while (temp != NULL) {
+      if ((type & temp->type) != 0) {
+        if (count == index)
+          return temp->flags;
+        count++;
+      }
+      temp = temp->next;
+    }
+  }
+  return 0;
+}
+
 /************************************************************************/
 /* Plugin initialization / deinitialization                             */
 /************************************************************************/
 
-void plugin_init_one(plugin_t *plugin)
+bool plugin_init_one(plugin_t *plugin)
 {
   /* initialize the plugin */
-  if (plugin->plugin_init(plugin, plugin->type))
+  if (plugin->plugin_entry(plugin, plugin->loadtype, PLUGIN_INIT))
   {
     pluginlog->info("Plugin initialization failed for %s", plugin->name);
-    plugin_abort();
+    plugin_abort(plugin);
+    return 0;
   }
-
   plugin->initialized = 1;
+  return 1;
 }
 
 
-plugin_t *plugin_unload(plugin_t *plugin)
+bool plugin_unload(plugin_t *plugin)
 {
-  plugin_t *dead_plug;
-
-  if (plugin->initialized)
-      plugin->plugin_fini();
-
+  if (plugin->loadtype != PLUGTYPE_NULL) {
+    if (plugin->initialized)
+      plugin->plugin_entry(plugin, plugin->type, PLUGIN_FINI);
 #if defined(WIN32)
-  FreeLibrary(plugin->handle);
+    FreeLibrary(plugin->handle);
 #else
-  lt_dlclose(plugin->handle);
+    lt_dlclose(plugin->handle);
 #endif
-  delete [] plugin->name;
-
-  dead_plug = plugin;
-  plugin = plugin->next;
-  delete dead_plug;
-
-  return plugin;
+    plugin->loadtype = PLUGTYPE_NULL;
+    return 1;
+  } else {
+    return 0;
+  }
 }
 
-void plugin_load(char *name, plugintype_t type)
+bool plugin_load(const char *name, Bit16u type)
 {
-  plugin_t *plugin, *temp;
+  plugin_t *plugin = NULL, *temp;
 #if defined(WIN32)
   char dll_path_list[MAX_PATH];
 #endif
@@ -306,27 +490,32 @@ void plugin_load(char *name, plugintype_t type)
     temp = plugins;
 
     while (temp != NULL) {
-      if (!strcmp(name, temp->name)) {
-        BX_PANIC(("plugin '%s' already loaded", name));
-        return;
+      if (!strcmp(name, temp->name) && ((type & temp->type) != 0)) {
+        if (temp->loadtype != PLUGTYPE_NULL) {
+          BX_PANIC(("plugin '%s' already loaded", name));
+          return 0;
+        } else {
+          plugin = temp;
+          break;
+        }
       }
       temp = temp->next;
     }
   }
-
-  plugin = new plugin_t;
-
-  plugin->type = type;
-  plugin->name = name;
-  plugin->initialized = 0;
+  if (plugin == NULL) {
+    BX_PANIC(("plugin '%s' not found", name));
+    return 0;
+  }
 
   char plugin_filename[BX_PATHNAME_LEN], tmpname[BX_PATHNAME_LEN];
-  if (type == PLUGTYPE_SOUND) {
-    sprintf(tmpname, SOUND_PLUGIN_FILENAME_FORMAT, name);
-  } else if (type == PLUGTYPE_NETWORK) {
+  if (type == PLUGTYPE_GUI) {
+    sprintf(tmpname, GUI_PLUGIN_FILENAME_FORMAT, name);
+  } else if (type == PLUGTYPE_IMG) {
+    sprintf(tmpname, IMG_PLUGIN_FILENAME_FORMAT, name);
+  } else if (type == PLUGTYPE_NET) {
     sprintf(tmpname, NET_PLUGIN_FILENAME_FORMAT, name);
-  } else if (type == PLUGTYPE_USBDEV) {
-    sprintf(tmpname, USB_PLUGIN_FILENAME_FORMAT, name);
+  } else if (type == PLUGTYPE_SND) {
+    sprintf(tmpname, SND_PLUGIN_FILENAME_FORMAT, name);
   } else {
     sprintf(tmpname, PLUGIN_FILENAME_FORMAT, name);
   }
@@ -334,7 +523,7 @@ void plugin_load(char *name, plugintype_t type)
 
   // Set context so that any devices that the plugin registers will
   // be able to see which plugin created them.  The registration will
-  // be called from either dlopen (global constructors) or plugin_init.
+  // be called from either dlopen (global constructors) or plugin_entry.
   BX_ASSERT(current_plugin_context == NULL);
   current_plugin_context = plugin;
 #if defined(WIN32)
@@ -355,8 +544,7 @@ void plugin_load(char *name, plugintype_t type)
     current_plugin_context = NULL;
     BX_PANIC(("LoadLibrary failed for module '%s' (%s): error=%d", name,
               plugin_filename, GetLastError()));
-    delete plugin;
-    return;
+    return 0;
   }
 #else
   plugin->handle = lt_dlopen(plugin_filename);
@@ -365,93 +553,56 @@ void plugin_load(char *name, plugintype_t type)
     current_plugin_context = NULL;
     BX_PANIC(("dlopen failed for module '%s' (%s): %s", name, plugin_filename,
               lt_dlerror()));
-    delete plugin;
-    return;
+    return 0;
   }
 #endif
+  plugin->loadtype = type;
 
   if (type == PLUGTYPE_GUI) {
-    sprintf(tmpname, GUI_PLUGIN_INIT_FMT_STRING, name);
-  } else if (type == PLUGTYPE_SOUND) {
-    sprintf(tmpname, SOUND_PLUGIN_INIT_FMT_STRING, name);
-  } else if (type == PLUGTYPE_NETWORK) {
-    sprintf(tmpname, NET_PLUGIN_INIT_FMT_STRING, name);
-  } else if (type == PLUGTYPE_USBDEV) {
-    sprintf(tmpname, USB_PLUGIN_INIT_FMT_STRING, name);
-  } else if (type != PLUGTYPE_USER) {
-    sprintf(tmpname, PLUGIN_INIT_FMT_STRING, name);
+    sprintf(tmpname, GUI_PLUGIN_ENTRY_FMT_STRING, name);
+  } else if (type == PLUGTYPE_IMG) {
+    sprintf(tmpname, IMG_PLUGIN_ENTRY_FMT_STRING, name);
+  } else if (type == PLUGTYPE_NET) {
+    sprintf(tmpname, NET_PLUGIN_ENTRY_FMT_STRING, name);
+  } else if (type == PLUGTYPE_SND) {
+    sprintf(tmpname, SND_PLUGIN_ENTRY_FMT_STRING, name);
   } else {
-    sprintf(tmpname, PLUGIN_INIT_FMT_STRING, "user");
+    sprintf(tmpname, PLUGIN_ENTRY_FMT_STRING, name);
   }
 #if defined(WIN32)
-  plugin->plugin_init = (plugin_init_t) GetProcAddress(plugin->handle, tmpname);
-  if (plugin->plugin_init == NULL) {
-    pluginlog->panic("could not find plugin_init: error=%d", GetLastError());
-    plugin_abort();
+  plugin->plugin_entry = (plugin_entry_t) GetProcAddress(plugin->handle, tmpname);
+  if (plugin->plugin_entry == NULL) {
+    pluginlog->panic("could not find plugin_entry for module '%s' (%s): error=%d",
+                     name, plugin_filename, GetLastError());
+    plugin_abort(plugin);
+    return 0;
   }
 #else
-  plugin->plugin_init = (plugin_init_t) lt_dlsym(plugin->handle, tmpname);
-  if (plugin->plugin_init == NULL) {
-    pluginlog->panic("could not find plugin_init: %s", lt_dlerror());
-    plugin_abort();
-  }
-#endif
-
-  if (type == PLUGTYPE_GUI) {
-    sprintf(tmpname, GUI_PLUGIN_FINI_FMT_STRING, name);
-  } else if (type == PLUGTYPE_SOUND) {
-    sprintf(tmpname, SOUND_PLUGIN_FINI_FMT_STRING, name);
-  } else if (type == PLUGTYPE_NETWORK) {
-    sprintf(tmpname, NET_PLUGIN_FINI_FMT_STRING, name);
-  } else if (type == PLUGTYPE_USBDEV) {
-    sprintf(tmpname, USB_PLUGIN_FINI_FMT_STRING, name);
-  } else if (type != PLUGTYPE_USER) {
-    sprintf(tmpname, PLUGIN_FINI_FMT_STRING, name);
-  } else {
-    sprintf(tmpname, PLUGIN_FINI_FMT_STRING, "user");
-  }
-#if defined(WIN32)
-  plugin->plugin_fini = (plugin_fini_t) GetProcAddress(plugin->handle, tmpname);
-  if (plugin->plugin_fini == NULL) {
-    pluginlog->panic("could not find plugin_fini: error=%d", GetLastError());
-    plugin_abort();
-  }
-#else
-  plugin->plugin_fini = (plugin_fini_t) lt_dlsym(plugin->handle, tmpname);
-  if (plugin->plugin_fini == NULL) {
-    pluginlog->panic("could not find plugin_fini: %s", lt_dlerror());
-    plugin_abort();
+  plugin->plugin_entry = (plugin_entry_t) lt_dlsym(plugin->handle, tmpname);
+  if (plugin->plugin_entry == NULL) {
+    pluginlog->panic("could not find plugin_entry for module '%s' (%s): %s",
+                     name, plugin_filename, lt_dlerror());
+    plugin_abort(plugin);
+    return 0;
   }
 #endif
   pluginlog->info("loaded plugin %s",plugin_filename);
 
-  /* Insert plugin at the _end_ of the plugin linked list. */
-  plugin->next = NULL;
-
-  if (!plugins) {
-    /* Empty list, this become the first entry. */
-    plugins = plugin;
-  } else {
-   /* Non-empty list.  Add to end. */
-   temp = plugins;
-
-   while (temp->next)
-      temp = temp->next;
-
-    temp->next = plugin;
+  if (!plugin_init_one(plugin)) {
+    return 0;
   }
 
-  plugin_init_one(plugin);
-
   // check that context didn't change.  This should only happen if we
   // need a reentrant plugin_load.
   BX_ASSERT(current_plugin_context == plugin);
   current_plugin_context = NULL;
+  return 1;
 }
 
-void plugin_abort(void)
+void plugin_abort(plugin_t *plugin)
 {
-  pluginlog->panic("plugin load aborted");
+  plugin_unload(plugin);
+  pluginlog->panic("loading plugin '%s' aborted", plugin->name);
 }
 
 #endif   /* end of #if BX_PLUGINS */
@@ -460,8 +611,7 @@ void plugin_abort(void)
 /* Plugin system: initialisation of plugins entry points                */
 /************************************************************************/
 
-  void
-plugin_startup(void)
+void plugin_startup(void)
 {
   pluginRegisterIRQ = builtinRegisterIRQ;
   pluginUnregisterIRQ = builtinUnregisterIRQ;
@@ -486,12 +636,33 @@ plugin_startup(void)
 
   pluginlog = new logfunctions();
   pluginlog->put("PLUGIN");
-#if BX_PLUGINS && !defined(WIN32)
+#if BX_PLUGINS
+#if !defined(WIN32)
   int status = lt_dlinit();
   if (status != 0) {
     BX_ERROR(("initialization error in ltdl library (for loading plugins)"));
     BX_PANIC(("error message was: %s", lt_dlerror()));
   }
+#endif
+  plugins_search();
+#endif
+}
+
+void plugin_cleanup(void)
+{
+#if BX_PLUGINS
+  plugin_t *dead_plug;
+
+  while (plugins != NULL) {
+    if (plugins->loadtype != PLUGTYPE_NULL) {
+      plugin_unload(plugins);
+    }
+    delete [] plugins->name;
+
+    dead_plug = plugins;
+    plugins = plugins->next;
+    delete dead_plug;
+  }
 #endif
 }
 
@@ -500,7 +671,7 @@ plugin_startup(void)
 /* Plugin system: Device registration                                   */
 /************************************************************************/
 
-void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmodel_c *devmodel, const char *name)
+void pluginRegisterDeviceDevmodel(plugin_t *plugin, Bit16u type, bx_devmodel_c *devmodel, const char *name)
 {
   device_t **devlist;
 
@@ -520,7 +691,6 @@ void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmod
       break;
     case PLUGTYPE_STANDARD:
     case PLUGTYPE_OPTIONAL:
-    case PLUGTYPE_USER:
     default:
       devlist = &devices;
       break;
@@ -544,14 +714,27 @@ void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmod
 /* Plugin system: Remove registered plugin device                       */
 /************************************************************************/
 
-void pluginUnregisterDeviceDevmodel(const char *name)
+void pluginUnregisterDeviceDevmodel(const char *name, Bit16u type)
 {
+  device_t **devlist;
   device_t *device, *prev = NULL;
 
-  for (device = devices; device; device = device->next) {
+  switch (type) {
+    case PLUGTYPE_CORE:
+    case PLUGTYPE_VGA:
+      devlist = &core_devices;
+      break;
+    case PLUGTYPE_STANDARD:
+    case PLUGTYPE_OPTIONAL:
+    default:
+      devlist = &devices;
+      break;
+  }
+
+  for (device = *devlist; device; device = device->next) {
     if (!strcmp(name, device->name)) {
       if (prev == NULL) {
-        devices = device->next;
+        *devlist = device->next;
       } else {
         prev->next = device->next;
       }
@@ -567,7 +750,7 @@ void pluginUnregisterDeviceDevmodel(const char *name)
 /* Plugin system: Check if a plugin is loaded                           */
 /************************************************************************/
 
-bx_bool pluginDevicePresent(const char *name)
+bool pluginDevicePresent(const char *name)
 {
   device_t *device;
 
@@ -584,32 +767,47 @@ bx_bool pluginDevicePresent(const char *name)
 /* Plugin system: Load one plugin                                       */
 /************************************************************************/
 
-int bx_load_plugin(const char *name, plugintype_t type)
+bool bx_load_plugin(const char *name, Bit16u type)
 {
-  char *namecopy = new char[1+strlen(name)];
-  strcpy(namecopy, name);
-  plugin_load(namecopy, type);
-  return 1;
+  plugin_t *plugin;
+
+  if (!strcmp(name, "*")) {
+    for (plugin = plugins; plugin; plugin = plugin->next) {
+      if (((type & plugin->type) != 0) && (plugin->loadtype == PLUGTYPE_NULL)) {
+        plugin_load(plugin->name, type);
+      }
+    }
+    return 1;
+  } else {
+    return plugin_load(name, type);
+  }
 }
 
-void bx_unload_plugin(const char *name, bx_bool devflag)
+bool bx_unload_plugin(const char *name, bool devflag)
 {
-  plugin_t *plugin, *prev = NULL;
+  plugin_t *plugin;
+  bool ret = 0;
 
   for (plugin = plugins; plugin; plugin = plugin->next) {
     if (!strcmp(plugin->name, name)) {
       if (devflag) {
-        pluginUnregisterDeviceDevmodel(plugin->name);
-      }
-      plugin = plugin_unload(plugin);
-      if (prev == NULL) {
-        plugins = plugin;
-      } else {
-        prev->next = plugin;
+        pluginUnregisterDeviceDevmodel(plugin->name, plugin->type);
       }
+      ret = plugin_unload(plugin);
+      break;
+    }
+  }
+  return ret;
+}
+
+void bx_unload_plugin_type(const char *name, Bit16u type)
+{
+  plugin_t *plugin;
+
+  for (plugin = plugins; plugin; plugin = plugin->next) {
+    if (!strcmp(plugin->name, name) && ((plugin->type & type) != 0)) {
+      plugin_unload(plugin);
       break;
-    } else {
-      prev = plugin;
     }
   }
 }
@@ -640,14 +838,6 @@ void bx_init_plugins()
       device->devmodel->init();
     }
   }
-#if BX_PLUGINS
-  for (device = devices; device; device = device->next) {
-    if (device->plugtype == PLUGTYPE_USER) {
-      pluginlog->info("init_dev of '%s' plugin device by virtual method",device->name);
-      device->devmodel->init();
-    }
-  }
-#endif
 }
 
 /**************************************************************************/
@@ -674,14 +864,6 @@ void bx_reset_plugins(unsigned signal)
       device->devmodel->reset(signal);
     }
   }
-#if BX_PLUGINS
-  for (device = devices; device; device = device->next) {
-    if (device->plugtype == PLUGTYPE_USER) {
-      pluginlog->info("reset of '%s' plugin device by virtual method",device->name);
-      device->devmodel->reset(signal);
-    }
-  }
-#endif
 }
 
 /*******************************************************/
@@ -692,6 +874,7 @@ void bx_unload_plugins()
 {
   device_t *device, *next;
 
+  // unload non-core plugins first
   device = devices;
   while (device != NULL) {
     if (device->plugin != NULL) {
@@ -710,12 +893,8 @@ void bx_unload_plugins()
     device = next;
   }
   devices = NULL;
-}
-
-void bx_unload_core_plugins()
-{
-  device_t *device, *next;
 
+  // now it's safe to unload core plugins
   device = core_devices;
   while (device != NULL) {
     if (device->plugin != NULL) {
@@ -771,36 +950,30 @@ void bx_plugins_after_restore_state()
       device->devmodel->after_restore_state();
     }
   }
-#if BX_PLUGINS
-  for (device = devices; device; device = device->next) {
-    if (device->plugtype == PLUGTYPE_USER) {
-      device->devmodel->after_restore_state();
-    }
-  }
-#endif
 }
 
 #if !BX_PLUGINS
 
-// Special code for loading gui, optional and sound plugins when plugin support
-// is turned off.
+// Special code for handling modules when plugin support is turned off.
 
-typedef struct {
-  const char*   name;
-  plugintype_t  type;
-  plugin_init_t plugin_init;
-  plugin_fini_t plugin_fini;
-  bx_bool       status;
-} builtin_plugin_t;
+#define BUILTIN_OPT_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_OPTIONAL, 0, lib##mod##_plugin_entry, 0}
+#define BUILTIN_OPTPCI_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_OPTIONAL, PLUGFLAG_PCI, lib##mod##_plugin_entry, 0}
+#define BUILTIN_VGA_PLUGIN_ENTRY(mod, t, f) {#mod, PLUGTYPE_VGA | t, f, lib##mod##_plugin_entry, 0}
+#define BUILTIN_CI_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_CI, 0, lib##mod##_plugin_entry, 0}
+#define BUILTIN_GUI_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_GUI, 0, lib##mod##_gui_plugin_entry, 0}
+#define BUILTIN_GUICI_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_GUI | PLUGTYPE_CI, 0, lib##mod##_gui_plugin_entry, 0}
+#define BUILTIN_IMG_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_IMG, 0, lib##mod##_img_plugin_entry, 0}
+#define BUILTIN_NET_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_NET, 0, libeth_##mod##_plugin_entry, 0}
+#define BUILTIN_SND_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_SND, 0, libsound##mod##_plugin_entry, 0}
+#define BUILTIN_USB_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_USB, 0, lib##mod##_plugin_entry, 0}
 
-#define BUILTIN_GUI_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_GUI, lib##mod##_gui_plugin_init, lib##mod##_gui_plugin_fini, 0}
-#define BUILTIN_OPT_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_OPTIONAL, lib##mod##_LTX_plugin_init, lib##mod##_LTX_plugin_fini, 0}
-#define BUILTIN_SND_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_SOUND, lib##mod##_sound_plugin_init, lib##mod##_sound_plugin_fini, 0}
-#define BUILTIN_NET_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_NETWORK, lib##mod##_net_plugin_init, lib##mod##_net_plugin_fini, 0}
-#define BUILTIN_USB_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_USBDEV, lib##mod##_dev_plugin_init, lib##mod##_dev_plugin_fini, 0}
-#define BUILTIN_VGA_PLUGIN_ENTRY(mod) {#mod, PLUGTYPE_VGA, lib##mod##_LTX_plugin_init, lib##mod##_LTX_plugin_fini, 0}
-
-static builtin_plugin_t builtin_plugins[] = {
+plugin_t bx_builtin_plugins[] = {
+#if BX_USE_TEXTCONFIG
+  BUILTIN_CI_PLUGIN_ENTRY(textconfig),
+#endif
+#if BX_USE_WIN32CONFIG
+  BUILTIN_CI_PLUGIN_ENTRY(win32config),
+#endif
 #if BX_WITH_AMIGAOS
   BUILTIN_GUI_PLUGIN_ENTRY(amigaos),
 #endif
@@ -822,9 +995,6 @@ static builtin_plugin_t builtin_plugins[] = {
 #if BX_WITH_SDL2
   BUILTIN_GUI_PLUGIN_ENTRY(sdl2),
 #endif
-#if BX_WITH_SVGA
-  BUILTIN_GUI_PLUGIN_ENTRY(svga),
-#endif
 #if BX_WITH_TERM
   BUILTIN_GUI_PLUGIN_ENTRY(term),
 #endif
@@ -835,10 +1005,17 @@ static builtin_plugin_t builtin_plugins[] = {
   BUILTIN_GUI_PLUGIN_ENTRY(win32),
 #endif
 #if BX_WITH_WX
-  BUILTIN_GUI_PLUGIN_ENTRY(wx),
+  BUILTIN_GUICI_PLUGIN_ENTRY(wx),
 #endif
 #if BX_WITH_X11
   BUILTIN_GUI_PLUGIN_ENTRY(x),
+#endif
+  BUILTIN_VGA_PLUGIN_ENTRY(vga, 0, PLUGFLAG_PCI),
+#if BX_SUPPORT_CLGD54XX
+  BUILTIN_VGA_PLUGIN_ENTRY(svga_cirrus, 0, PLUGFLAG_PCI),
+#endif
+#if BX_SUPPORT_VOODOO
+  BUILTIN_VGA_PLUGIN_ENTRY(voodoo, PLUGTYPE_OPTIONAL, PLUGFLAG_PCI),
 #endif
   BUILTIN_OPT_PLUGIN_ENTRY(unmapped),
   BUILTIN_OPT_PLUGIN_ENTRY(biosdev),
@@ -850,10 +1027,10 @@ static builtin_plugin_t builtin_plugins[] = {
   BUILTIN_OPT_PLUGIN_ENTRY(busmouse),
 #endif
 #if BX_SUPPORT_E1000
-  BUILTIN_OPT_PLUGIN_ENTRY(e1000),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(e1000),
 #endif
 #if BX_SUPPORT_ES1370
-  BUILTIN_OPT_PLUGIN_ENTRY(es1370),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(es1370),
 #endif
 #if BX_SUPPORT_GAMEPORT
   BUILTIN_OPT_PLUGIN_ENTRY(gameport),
@@ -862,32 +1039,28 @@ static builtin_plugin_t builtin_plugins[] = {
   BUILTIN_OPT_PLUGIN_ENTRY(iodebug),
 #endif
 #if BX_SUPPORT_NE2K
-  BUILTIN_OPT_PLUGIN_ENTRY(ne2k),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(ne2k),
 #endif
 #if BX_SUPPORT_PCIDEV
-  BUILTIN_OPT_PLUGIN_ENTRY(pcidev),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(pcidev),
 #endif
 #if BX_SUPPORT_PCIPNIC
-  BUILTIN_OPT_PLUGIN_ENTRY(pcipnic),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(pcipnic),
 #endif
 #if BX_SUPPORT_SB16
   BUILTIN_OPT_PLUGIN_ENTRY(sb16),
 #endif
 #if BX_SUPPORT_USB_UHCI
-  BUILTIN_OPT_PLUGIN_ENTRY(usb_uhci),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(usb_uhci),
 #endif
 #if BX_SUPPORT_USB_OHCI
-  BUILTIN_OPT_PLUGIN_ENTRY(usb_ohci),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(usb_ohci),
 #endif
 #if BX_SUPPORT_USB_EHCI
-  BUILTIN_OPT_PLUGIN_ENTRY(usb_ehci),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(usb_ehci),
 #endif
 #if BX_SUPPORT_USB_XHCI
-  BUILTIN_OPT_PLUGIN_ENTRY(usb_xhci),
-#endif
-#if BX_SUPPORT_VOODOO
-  BUILTIN_VGA_PLUGIN_ENTRY(voodoo),
-  BUILTIN_OPT_PLUGIN_ENTRY(voodoo),
+  BUILTIN_OPTPCI_PLUGIN_ENTRY(usb_xhci),
 #endif
 #if BX_SUPPORT_SOUNDLOW
   BUILTIN_SND_PLUGIN_ENTRY(dummy),
@@ -909,13 +1082,14 @@ static builtin_plugin_t builtin_plugins[] = {
 #endif
 #endif
 #if BX_NETWORKING
+  BUILTIN_NET_PLUGIN_ENTRY(null),
+  BUILTIN_NET_PLUGIN_ENTRY(vnet),
 #if BX_NETMOD_FBSD
   BUILTIN_NET_PLUGIN_ENTRY(fbsd),
 #endif
 #if BX_NETMOD_LINUX
   BUILTIN_NET_PLUGIN_ENTRY(linux),
 #endif
-  BUILTIN_NET_PLUGIN_ENTRY(null),
 #if BX_NETMOD_SLIRP
   BUILTIN_NET_PLUGIN_ENTRY(slirp),
 #endif
@@ -931,55 +1105,108 @@ static builtin_plugin_t builtin_plugins[] = {
 #if BX_NETMOD_VDE
   BUILTIN_NET_PLUGIN_ENTRY(vde),
 #endif
-  BUILTIN_NET_PLUGIN_ENTRY(vnet),
 #if BX_NETMOD_WIN32
   BUILTIN_NET_PLUGIN_ENTRY(win32),
 #endif
 #endif
 #if BX_SUPPORT_PCIUSB
-  BUILTIN_USB_PLUGIN_ENTRY(usb_cbi),
+  BUILTIN_USB_PLUGIN_ENTRY(usb_floppy),
   BUILTIN_USB_PLUGIN_ENTRY(usb_hid),
   BUILTIN_USB_PLUGIN_ENTRY(usb_hub),
   BUILTIN_USB_PLUGIN_ENTRY(usb_msd),
   BUILTIN_USB_PLUGIN_ENTRY(usb_printer),
 #endif
-  {"NULL", PLUGTYPE_GUI, NULL, NULL, 0}
+  BUILTIN_IMG_PLUGIN_ENTRY(vmware3),
+  BUILTIN_IMG_PLUGIN_ENTRY(vmware4),
+  BUILTIN_IMG_PLUGIN_ENTRY(vbox),
+  BUILTIN_IMG_PLUGIN_ENTRY(vpc),
+  BUILTIN_IMG_PLUGIN_ENTRY(vvfat),
+  {"NULL", PLUGTYPE_NULL, 0, NULL, 0}
 };
 
-int bx_load_plugin2(const char *name, plugintype_t type)
+Bit8u bx_get_plugins_count_np(Bit16u type)
 {
   int i = 0;
-  while (strcmp(builtin_plugins[i].name, "NULL")) {
-    if ((!strcmp(name, builtin_plugins[i].name)) &&
-        (type == builtin_plugins[i].type)) {
-      if (builtin_plugins[i].status == 0) {
-        builtin_plugins[i].plugin_init(NULL, type);
-        builtin_plugins[i].status = 1;
-      }
-      return 1;
+  Bit8u count = 0;
+
+  while (strcmp(bx_builtin_plugins[i].name, "NULL")) {
+    if ((type & bx_builtin_plugins[i].type) != 0)
+      count++;
+    i++;
+  }
+  return count;
+}
+
+const char* bx_get_plugin_name_np(Bit16u type, Bit8u index)
+{
+  int i = 0;
+  Bit8u count = 0;
+
+  while (strcmp(bx_builtin_plugins[i].name, "NULL")) {
+    if ((type & bx_builtin_plugins[i].type) != 0) {
+      if (count == index)
+        return bx_builtin_plugins[i].name;
+      count++;
     }
     i++;
-  };
+  }
+  return NULL;
+}
+
+Bit8u bx_get_plugin_flags_np(Bit16u type, Bit8u index)
+{
+  int i = 0;
+  Bit8u count = 0;
+
+  while (strcmp(bx_builtin_plugins[i].name, "NULL")) {
+    if ((type & bx_builtin_plugins[i].type) != 0) {
+      if (count == index)
+        return bx_builtin_plugins[i].flags;
+      count++;
+    }
+    i++;
+  }
   return 0;
 }
 
-int bx_unload_opt_plugin(const char *name, bx_bool devflag)
+int bx_load_plugin_np(const char *name, Bit16u type)
 {
   int i = 0;
-  while (strcmp(builtin_plugins[i].name, "NULL")) {
-    if ((!strcmp(name, builtin_plugins[i].name)) &&
-        (builtin_plugins[i].type == PLUGTYPE_OPTIONAL)) {
-      if (builtin_plugins[i].status == 1) {
-        if (devflag) {
-          pluginUnregisterDeviceDevmodel(builtin_plugins[i].name);
-        }
-        builtin_plugins[i].plugin_fini();
-        builtin_plugins[i].status = 0;
+  while (strcmp(bx_builtin_plugins[i].name, "NULL")) {
+    if ((!strcmp(name, bx_builtin_plugins[i].name)) &&
+        ((type & bx_builtin_plugins[i].type) != 0)) {
+      if (bx_builtin_plugins[i].initialized == 0) {
+        bx_builtin_plugins[i].loadtype = type;
+        bx_builtin_plugins[i].plugin_entry(NULL, type, PLUGIN_INIT);
+        bx_builtin_plugins[i].initialized = 1;
       }
       return 1;
     }
     i++;
-  };
+  }
+  return 0;
+}
+
+int bx_unload_opt_plugin(const char *name, bool devflag)
+{
+  int i = 0;
+  while (strcmp(bx_builtin_plugins[i].name, "NULL")) {
+    if ((!strcmp(name, bx_builtin_plugins[i].name)) &&
+        ((bx_builtin_plugins[i].type == PLUGTYPE_OPTIONAL) ||
+         (bx_builtin_plugins[i].type == PLUGTYPE_VGA))) {
+      if (bx_builtin_plugins[i].initialized == 1) {
+        if (devflag) {
+          pluginUnregisterDeviceDevmodel(bx_builtin_plugins[i].name,
+                                         bx_builtin_plugins[i].type);
+        }
+        bx_builtin_plugins[i].plugin_entry(NULL, bx_builtin_plugins[i].loadtype, PLUGIN_FINI);
+        bx_builtin_plugins[i].loadtype = PLUGTYPE_NULL;
+        bx_builtin_plugins[i].initialized = 0;
+      }
+      return 1;
+    }
+    i++;
+  }
   return 0;
 }
 
