@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: plugin.h 13504 2018-05-10 10:50:42Z vruppert $
+// $Id: plugin.h 14293 2021-06-27 14:50:26Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2018  The Bochs Project
+//  Copyright (C) 2002-2021  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,7 @@
 
 #include "extplugin.h"
 
+class BX_MEM_C;
 class bx_devices_c;
 BOCHSAPI extern logfunctions  *pluginlog;
 
@@ -50,7 +51,6 @@ extern "C" {
 #define BX_PLUGIN_KEYBOARD  "keyboard"
 #define BX_PLUGIN_BUSMOUSE  "busmouse"
 #define BX_PLUGIN_HARDDRV   "harddrv"
-#define BX_PLUGIN_HDIMAGE   "hdimage"
 #define BX_PLUGIN_DMA       "dma"
 #define BX_PLUGIN_PIC       "pic"
 #define BX_PLUGIN_PIT       "pit"
@@ -78,25 +78,21 @@ extern "C" {
 
 
 #define BX_REGISTER_DEVICE_DEVMODEL(a,b,c,d) pluginRegisterDeviceDevmodel(a,b,c,d)
-#define BX_UNREGISTER_DEVICE_DEVMODEL(a) pluginUnregisterDeviceDevmodel(a)
+#define BX_UNREGISTER_DEVICE_DEVMODEL(a,b) pluginUnregisterDeviceDevmodel(a,b)
 #define PLUG_device_present(a) pluginDevicePresent(a)
 
 #if BX_PLUGINS
 
+// hardcoded load plugin macro for PLUGTYPE_CORE and PLUGTYPE_STANDARD
 #define PLUG_load_plugin(name,type) {bx_load_plugin(#name,type);}
-#define PLUG_load_gui_plugin(name) bx_load_plugin(name,PLUGTYPE_GUI)
+// newer plugin macros for variable plugin handling
+#define PLUG_get_plugins_count(type) bx_get_plugins_count(type)
+#define PLUG_get_plugin_name(type,index) bx_get_plugin_name(type,index)
+#define PLUG_get_plugin_flags(type,index) bx_get_plugin_flags(type,index)
+#define PLUG_load_plugin_var(name,type) {bx_load_plugin(name,type);}
 #define PLUG_load_opt_plugin(name) bx_load_plugin(name,PLUGTYPE_OPTIONAL)
-#define PLUG_load_snd_plugin(name) bx_load_plugin(name,PLUGTYPE_SOUND)
-#define PLUG_load_net_plugin(name) bx_load_plugin(name,PLUGTYPE_NETWORK)
-#define PLUG_load_usb_plugin(name) bx_load_plugin(name,PLUGTYPE_USBDEV)
-#define PLUG_load_vga_plugin(name) bx_load_plugin(name,PLUGTYPE_VGA)
-#define PLUG_load_user_plugin(name) {bx_load_plugin(name,PLUGTYPE_USER);}
-#define PLUG_unload_plugin(name) {bx_unload_plugin(#name,1);}
 #define PLUG_unload_opt_plugin(name) bx_unload_plugin(name,1)
-#define PLUG_unload_snd_plugin(name) bx_unload_plugin(name,0)
-#define PLUG_unload_net_plugin(name) bx_unload_plugin(name,0)
-#define PLUG_unload_usb_plugin(name) bx_unload_plugin(name,0)
-#define PLUG_unload_user_plugin(name) {bx_unload_plugin(name,1);}
+#define PLUG_unload_plugin_type(name,type) {bx_unload_plugin_type(name,type);}
 
 #define DEV_register_ioread_handler(b,c,d,e,f)  pluginRegisterIOReadHandler(b,c,d,e,f)
 #define DEV_register_iowrite_handler(b,c,d,e,f) pluginRegisterIOWriteHandler(b,c,d,e,f)
@@ -114,17 +110,16 @@ extern "C" {
 
 #else
 
-// When plugins are off, PLUG_load_plugin will call the plugin_init function
-// directly.
-#define PLUG_load_plugin(name,type) {lib##name##_LTX_plugin_init(NULL,type);}
-#define PLUG_load_gui_plugin(name) bx_load_plugin2(name,PLUGTYPE_GUI)
-#define PLUG_load_opt_plugin(name) bx_load_plugin2(name,PLUGTYPE_OPTIONAL)
-#define PLUG_load_snd_plugin(name) bx_load_plugin2(name,PLUGTYPE_SOUND)
-#define PLUG_load_net_plugin(name) bx_load_plugin2(name,PLUGTYPE_NETWORK)
-#define PLUG_load_usb_plugin(name) bx_load_plugin2(name,PLUGTYPE_USBDEV)
-#define PLUG_load_vga_plugin(name) bx_load_plugin2(name,PLUGTYPE_VGA)
-#define PLUG_unload_plugin(name) {lib##name##_LTX_plugin_fini();}
-#define PLUG_unload_opt_plugin(name) bx_unload_opt_plugin(name,1);
+// When plugins are off, PLUG_load_plugin will call the plugin_entry function
+// directly (PLUGTYPE_CORE and PLUGTYPE_STANDARD only).
+#define PLUG_load_plugin(name,type) {lib##name##_plugin_entry(NULL,type,PLUGIN_INIT);}
+// Builtin plugins macros
+#define PLUG_get_plugins_count(type) bx_get_plugins_count_np(type)
+#define PLUG_get_plugin_name(type,index) bx_get_plugin_name_np(type,index)
+#define PLUG_get_plugin_flags(type,index) bx_get_plugin_flags_np(type,index)
+#define PLUG_load_plugin_var(name,type) bx_load_plugin_np(name,type)
+#define PLUG_load_opt_plugin(name) bx_load_plugin_np(name,PLUGTYPE_OPTIONAL)
+#define PLUG_unload_opt_plugin(name) bx_unload_opt_plugin(name,1)
 
 #define DEV_register_ioread_handler(b,c,d,e,f) bx_devices.register_io_read_handler(b,c,d,e,f)
 #define DEV_register_iowrite_handler(b,c,d,e,f) bx_devices.register_io_write_handler(b,c,d,e,f)
@@ -149,8 +144,8 @@ extern "C" {
 #define DEV_register_timer(a,b,c,d,e,f) bx_pc_system.register_timer(a,b,c,d,e,f)
 
 ///////// Removable devices macros
-#define DEV_optional_key_enq(a) (bx_devices.optional_key_enq(a))
-#define DEV_register_removable_keyboard(a,b) (bx_devices.register_removable_keyboard(a,b))
+#define DEV_register_default_keyboard(a,b,c) (bx_devices.register_default_keyboard(a,b,c))
+#define DEV_register_removable_keyboard(a,b,c,d) (bx_devices.register_removable_keyboard(a,b,c,d))
 #define DEV_unregister_removable_keyboard(a) (bx_devices.unregister_removable_keyboard(a))
 #define DEV_register_default_mouse(a,b,c) (bx_devices.register_default_mouse(a,b,c))
 #define DEV_register_removable_mouse(a,b,c) (bx_devices.register_removable_mouse(a,b,c))
@@ -173,12 +168,11 @@ extern "C" {
 
 ///////// keyboard macros
 #define DEV_kbd_gen_scancode(key) (bx_devices.gen_scancode(key))
-#define DEV_kbd_paste_bytes(bytes, count) \
-    (bx_devices.pluginKeyboard->paste_bytes(bytes,count))
+#define DEV_kbd_paste_bytes(bytes, count) (bx_devices.paste_bytes(bytes,count))
 #define DEV_kbd_release_keys() (bx_devices.release_keys())
+#define DEV_kbd_set_indicator(a,b,c) (bx_devices.kbd_set_indicator(a,b,c))
 
-///////// mouse macros
-#define DEV_mouse_enabled_changed(en) (bx_devices.mouse_enabled_changed(en))
+///////// mouse macro
 #define DEV_mouse_motion(dx, dy, dz, bs, absxy) (bx_devices.mouse_motion(dx, dy, dz, bs, absxy))
 
 ///////// hard drive macros
@@ -186,25 +180,14 @@ extern "C" {
     (bx_devices.pluginHardDrive->virt_read_handler(b, c))
 #define DEV_hd_write_handler(a, b, c, d) \
     (bx_devices.pluginHardDrive->virt_write_handler(b, c, d))
-#define DEV_hd_get_first_cd_handle() \
-    (bx_devices.pluginHardDrive->get_first_cd_handle())
-#define DEV_hd_get_cd_media_status(handle) \
-    (bx_devices.pluginHardDrive->get_cd_media_status(handle))
-#define DEV_hd_set_cd_media_status(handle, status) \
-    (bx_devices.pluginHardDrive->set_cd_media_status(handle, status))
 #define DEV_hd_bmdma_read_sector(a,b,c) bx_devices.pluginHardDrive->bmdma_read_sector(a,b,c)
 #define DEV_hd_bmdma_write_sector(a,b) bx_devices.pluginHardDrive->bmdma_write_sector(a,b)
 #define DEV_hd_bmdma_complete(a) bx_devices.pluginHardDrive->bmdma_complete(a)
-#define DEV_hdimage_init_image(a,b,c) bx_devices.pluginHDImageCtl->init_image(a,b,c)
-#define DEV_hdimage_init_cdrom(a) bx_devices.pluginHDImageCtl->init_cdrom(a)
 
 #define DEV_bulk_io_quantum_requested() (bx_devices.bulkIOQuantumsRequested)
 #define DEV_bulk_io_quantum_transferred() (bx_devices.bulkIOQuantumsTransferred)
 #define DEV_bulk_io_host_addr() (bx_devices.bulkIOHostAddr)
 
-///////// FLOPPY macro
-#define DEV_floppy_set_media_status(drive, status)  bx_devices.pluginFloppyDevice->set_media_status(drive, status)
-
 ///////// DMA macros
 #define DEV_dma_register_8bit_channel(channel, dmaRead, dmaWrite, name) \
   (bx_devices.pluginDmaDevice->registerDMA8Channel(channel, dmaRead, dmaWrite, name))
@@ -226,8 +209,6 @@ extern "C" {
 #define DEV_pic_iac()         (bx_devices.pluginPicDevice->IAC())
 
 ///////// VGA macros
-#define DEV_vga_mem_read(addr) (bx_devices.pluginVgaDevice->mem_read(addr))
-#define DEV_vga_mem_write(addr, val) (bx_devices.pluginVgaDevice->mem_write(addr, val))
 #define DEV_vga_redraw_area(left, top, right, bottom) \
   (bx_devices.pluginVgaDevice->vga_redraw_area(left, top, right, bottom))
 #define DEV_vga_get_text_snapshot(rawsnap, height, width) \
@@ -241,6 +222,7 @@ extern "C" {
   (bx_devices.register_pci_handlers(a,b,c,d,0))
 #define DEV_register_pci_handlers2(a,b,c,d,e) \
   (bx_devices.register_pci_handlers(a,b,c,d,e))
+#define DEV_pci_get_slot_mapping() bx_devices.pci_get_slot_mapping()
 #define DEV_pci_get_confAddr() bx_devices.pci_get_confAddr()
 #define DEV_pci_set_irq(a,b,c) bx_devices.pluginPci2IsaBridge->pci_set_irq(a,b,c)
 #define DEV_pci_set_base_mem(a,b,c,d,e,f) \
@@ -252,6 +234,7 @@ extern "C" {
 #define DEV_ide_bmdma_start_transfer(a) \
   bx_devices.pluginPciIdeController->bmdma_start_transfer(a)
 #define DEV_acpi_generate_smi(a) bx_devices.pluginACPIController->generate_smi(a)
+#define DEV_agp_present() (bx_devices.is_agp_present())
 
 ///////// Speaker macros
 #define DEV_speaker_beep_on(frequency) bx_devices.pluginSpeaker->beep_on(frequency)
@@ -266,9 +249,10 @@ extern "C" {
 #define DEV_mem_set_memory_type(a,b,c) \
     bx_devices.mem->set_memory_type((memory_area_t)a,b,c)
 #define DEV_mem_set_bios_write(a) bx_devices.mem->set_bios_write(a)
+#define DEV_mem_set_bios_rom_access(a,b) bx_devices.mem->set_bios_rom_access(a,b)
 
 ///////// USB device macro
-#define DEV_usb_init_device(a,b,c,d) (usbdev_type)bx_usbdev_ctl.init_device(a,b,(void**)c,d)
+#define DEV_usb_init_device(a,b,c) bx_usbdev_ctl.init_device(a,b,(void**)c)
 
 ///////// Sound module macros
 #define DEV_sound_get_waveout(a) (bx_soundmod_ctl.get_waveout(a))
@@ -294,13 +278,11 @@ extern "C" {
 typedef Bit32u (*ioReadHandler_t)(void *, Bit32u, unsigned);
 typedef void   (*ioWriteHandler_t)(void *, Bit32u, Bit32u, unsigned);
 
-extern plugin_t *plugins;
-
 typedef struct _device_t
 {
     const char   *name;
     plugin_t     *plugin;
-    plugintype_t plugtype;
+    Bit16u       plugtype;
 
     class bx_devmodel_c *devmodel;  // BBD hack
 
@@ -311,15 +293,16 @@ typedef struct _device_t
 extern device_t *devices;
 
 void plugin_startup(void);
+void plugin_cleanup(void);
 
 /* === Device Stuff === */
 typedef void (*deviceInitMem_t)(BX_MEM_C *);
 typedef void (*deviceInitDev_t)(void);
 typedef void (*deviceReset_t)(unsigned);
 
-BOCHSAPI void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmodel_c *dev, const char *name);
-BOCHSAPI void pluginUnregisterDeviceDevmodel(const char *name);
-BOCHSAPI bx_bool pluginDevicePresent(const char *name);
+BOCHSAPI void pluginRegisterDeviceDevmodel(plugin_t *plugin, Bit16u type, bx_devmodel_c *dev, const char *name);
+BOCHSAPI void pluginUnregisterDeviceDevmodel(const char *name, Bit16u type);
+BOCHSAPI bool pluginDevicePresent(const char *name);
 
 /* === IO port stuff === */
 BOCHSAPI extern int (*pluginRegisterIOReadHandler)(void *thisPtr, ioReadHandler_t callback,
@@ -351,141 +334,162 @@ BOCHSAPI extern void  (*pluginUnregisterIRQ)(unsigned irq, const char *name);
 BOCHSAPI extern void    (*pluginSetHRQ)(unsigned val);
 BOCHSAPI extern void    (*pluginSetHRQHackCallback)(void (*callback)(void));
 
-void plugin_abort(void);
+void plugin_abort(plugin_t *plugin);
 
-int bx_load_plugin(const char *name, plugintype_t type);
-extern void bx_unload_plugin(const char *name, bx_bool devflag);
+#if BX_PLUGINS
+Bit8u bx_get_plugins_count(Bit16u type);
+const char* bx_get_plugin_name(Bit16u type, Bit8u index);
+Bit8u bx_get_plugin_flags(Bit16u type, Bit8u index);
+#endif
+bool bx_load_plugin(const char *name, Bit16u type);
+bool bx_unload_plugin(const char *name, bool devflag);
+extern void bx_unload_plugin_type(const char *name, Bit16u type);
 extern void bx_init_plugins(void);
 extern void bx_reset_plugins(unsigned);
 extern void bx_unload_plugins(void);
-extern void bx_unload_core_plugins(void);
 extern void bx_plugins_register_state(void);
 extern void bx_plugins_after_restore_state(void);
 
 #if !BX_PLUGINS
-int bx_load_plugin2(const char *name, plugintype_t type);
-int bx_unload_opt_plugin(const char *name, bx_bool devflag);
+extern plugin_t bx_builtin_plugins[];
+
+Bit8u bx_get_plugins_count_np(Bit16u type);
+const char* bx_get_plugin_name_np(Bit16u type, Bit8u index);
+Bit8u bx_get_plugin_flags_np(Bit16u type, Bit8u index);
+int bx_load_plugin_np(const char *name, Bit16u type);
+int bx_unload_opt_plugin(const char *name, bool devflag);
 #endif
 
-// every plugin must define these, within the extern"C" block, so that
+// every plugin must define this, within the extern"C" block, so that
 // a non-mangled function symbol is available in the shared library.
-void plugin_fini(void);
-int plugin_init(plugin_t *plugin, plugintype_t type);
+int plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode);
 
 // still in extern "C"
-#if BX_PLUGINS && defined(_MSC_VER)
-#define DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(mod) \
-  extern "C" __declspec(dllexport) int __cdecl lib##mod##_LTX_plugin_init(plugin_t *plugin, plugintype_t type); \
-  extern "C" __declspec(dllexport) void __cdecl lib##mod##_LTX_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(mod) \
-  extern "C" __declspec(dllexport) int __cdecl lib##mod##_gui_plugin_init(plugin_t *plugin, plugintype_t type); \
-  extern "C" __declspec(dllexport) void __cdecl lib##mod##_gui_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(mod) \
-  extern "C" __declspec(dllexport) int __cdecl lib##mod##_sound_plugin_init(plugin_t *plugin, plugintype_t type); \
-  extern "C" __declspec(dllexport) void __cdecl lib##mod##_sound_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(mod) \
-  extern "C" __declspec(dllexport) int __cdecl lib##mod##_net_plugin_init(plugin_t *plugin, plugintype_t type); \
-  extern "C" __declspec(dllexport) void __cdecl lib##mod##_net_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(mod) \
-  extern "C" __declspec(dllexport) int __cdecl lib##mod##_dev_plugin_init(plugin_t *plugin, plugintype_t type); \
-  extern "C" __declspec(dllexport) void __cdecl lib##mod##_dev_plugin_fini(void);
+#if BX_PLUGINS && defined(WIN32)
+
+#define PLUGIN_ENTRY_FOR_MODULE(mod) \
+  extern "C" __declspec(dllexport) int __cdecl lib##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_GUI_MODULE(mod) \
+  extern "C" __declspec(dllexport) int __cdecl lib##mod##_gui_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_IMG_MODULE(mod) \
+  extern "C" __declspec(dllexport) int __cdecl lib##mod##_img_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_NET_MODULE(mod) \
+  extern "C" __declspec(dllexport) int __cdecl libeth_##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_SND_MODULE(mod) \
+  extern "C" __declspec(dllexport) int __cdecl libsound##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+
+#elif BX_PLUGINS
+
+#define PLUGIN_ENTRY_FOR_MODULE(mod) \
+  extern "C" int CDECL lib##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_GUI_MODULE(mod) \
+  extern "C" int CDECL lib##mod##_gui_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_IMG_MODULE(mod) \
+  extern "C" int CDECL lib##mod##_img_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_NET_MODULE(mod) \
+  extern "C" int CDECL libeth_##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_SND_MODULE(mod) \
+  extern "C" int CDECL libsound##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+
 #else
-#define DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(mod) \
-  int CDECL lib##mod##_LTX_plugin_init(plugin_t *plugin, plugintype_t type); \
-  void CDECL lib##mod##_LTX_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(mod) \
-  int CDECL lib##mod##_gui_plugin_init(plugin_t *plugin, plugintype_t type); \
-  void CDECL lib##mod##_gui_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(mod) \
-  int CDECL lib##mod##_sound_plugin_init(plugin_t *plugin, plugintype_t type); \
-  void CDECL lib##mod##_sound_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(mod) \
-  int CDECL lib##mod##_net_plugin_init(plugin_t *plugin, plugintype_t type); \
-  void CDECL lib##mod##_net_plugin_fini(void);
-#define DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(mod) \
-  int CDECL lib##mod##_dev_plugin_init(plugin_t *plugin, plugintype_t type); \
-  void CDECL lib##mod##_dev_plugin_fini(void);
-#endif
+
+#define PLUGIN_ENTRY_FOR_MODULE(mod) \
+  int CDECL lib##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_GUI_MODULE(mod) \
+  int CDECL lib##mod##_gui_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_IMG_MODULE(mod) \
+  int CDECL lib##mod##_img_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_NET_MODULE(mod) \
+  int CDECL libeth_##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
+#define PLUGIN_ENTRY_FOR_SND_MODULE(mod) \
+  int CDECL libsound##mod##_plugin_entry(plugin_t *plugin, Bit16u type, Bit8u mode)
 
 // device plugins
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(harddrv)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(hdimage)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(keyboard)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(busmouse)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(serial)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(unmapped)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(biosdev)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(cmos)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(dma)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pic)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pit)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(vga)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(svga_cirrus)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(floppy)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(parallel)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pci)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pci2isa)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pci_ide)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pcidev)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(usb_uhci)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(usb_ohci)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(usb_ehci)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(usb_xhci)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(sb16)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(es1370)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(netmod)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(ne2k)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(pcipnic)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(e1000)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(extfpuirq)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(gameport)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(speaker)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(acpi)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(iodebug)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(ioapic)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(hpet)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(voodoo)
-DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(user)
+PLUGIN_ENTRY_FOR_MODULE(harddrv);
+PLUGIN_ENTRY_FOR_MODULE(keyboard);
+PLUGIN_ENTRY_FOR_MODULE(busmouse);
+PLUGIN_ENTRY_FOR_MODULE(serial);
+PLUGIN_ENTRY_FOR_MODULE(unmapped);
+PLUGIN_ENTRY_FOR_MODULE(biosdev);
+PLUGIN_ENTRY_FOR_MODULE(cmos);
+PLUGIN_ENTRY_FOR_MODULE(dma);
+PLUGIN_ENTRY_FOR_MODULE(pic);
+PLUGIN_ENTRY_FOR_MODULE(pit);
+PLUGIN_ENTRY_FOR_MODULE(vga);
+PLUGIN_ENTRY_FOR_MODULE(svga_cirrus);
+PLUGIN_ENTRY_FOR_MODULE(floppy);
+PLUGIN_ENTRY_FOR_MODULE(parallel);
+PLUGIN_ENTRY_FOR_MODULE(pci);
+PLUGIN_ENTRY_FOR_MODULE(pci2isa);
+PLUGIN_ENTRY_FOR_MODULE(pci_ide);
+PLUGIN_ENTRY_FOR_MODULE(pcidev);
+PLUGIN_ENTRY_FOR_MODULE(usb_uhci);
+PLUGIN_ENTRY_FOR_MODULE(usb_ohci);
+PLUGIN_ENTRY_FOR_MODULE(usb_ehci);
+PLUGIN_ENTRY_FOR_MODULE(usb_xhci);
+PLUGIN_ENTRY_FOR_MODULE(sb16);
+PLUGIN_ENTRY_FOR_MODULE(es1370);
+PLUGIN_ENTRY_FOR_MODULE(netmod);
+PLUGIN_ENTRY_FOR_MODULE(ne2k);
+PLUGIN_ENTRY_FOR_MODULE(pcipnic);
+PLUGIN_ENTRY_FOR_MODULE(e1000);
+PLUGIN_ENTRY_FOR_MODULE(extfpuirq);
+PLUGIN_ENTRY_FOR_MODULE(gameport);
+PLUGIN_ENTRY_FOR_MODULE(speaker);
+PLUGIN_ENTRY_FOR_MODULE(acpi);
+PLUGIN_ENTRY_FOR_MODULE(iodebug);
+PLUGIN_ENTRY_FOR_MODULE(ioapic);
+PLUGIN_ENTRY_FOR_MODULE(hpet);
+PLUGIN_ENTRY_FOR_MODULE(voodoo);
+// config interface plugins
+PLUGIN_ENTRY_FOR_MODULE(textconfig);
+PLUGIN_ENTRY_FOR_MODULE(win32config);
 // gui plugins
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(amigaos)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(carbon)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(macintosh)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(nogui)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(rfb)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(sdl)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(sdl2)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(svga)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(term)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(vncsrv)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(win32)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(wx)
-DECLARE_PLUGIN_INIT_FINI_FOR_GUI_MODULE(x)
+PLUGIN_ENTRY_FOR_GUI_MODULE(amigaos);
+PLUGIN_ENTRY_FOR_GUI_MODULE(carbon);
+PLUGIN_ENTRY_FOR_GUI_MODULE(macintosh);
+PLUGIN_ENTRY_FOR_GUI_MODULE(nogui);
+PLUGIN_ENTRY_FOR_GUI_MODULE(rfb);
+PLUGIN_ENTRY_FOR_GUI_MODULE(sdl);
+PLUGIN_ENTRY_FOR_GUI_MODULE(sdl2);
+PLUGIN_ENTRY_FOR_GUI_MODULE(term);
+PLUGIN_ENTRY_FOR_GUI_MODULE(vncsrv);
+PLUGIN_ENTRY_FOR_GUI_MODULE(win32);
+PLUGIN_ENTRY_FOR_GUI_MODULE(wx);
+PLUGIN_ENTRY_FOR_GUI_MODULE(x);
 // sound driver plugins
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(alsa)
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(dummy)
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(file)
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(oss)
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(osx)
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(sdl)
-DECLARE_PLUGIN_INIT_FINI_FOR_SOUND_MODULE(win)
+PLUGIN_ENTRY_FOR_SND_MODULE(alsa);
+PLUGIN_ENTRY_FOR_SND_MODULE(dummy);
+PLUGIN_ENTRY_FOR_SND_MODULE(file);
+PLUGIN_ENTRY_FOR_SND_MODULE(oss);
+PLUGIN_ENTRY_FOR_SND_MODULE(osx);
+PLUGIN_ENTRY_FOR_SND_MODULE(sdl);
+PLUGIN_ENTRY_FOR_SND_MODULE(win);
 // network driver plugins
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(fbsd)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(linux)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(null)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(slirp)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(socket)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(tap)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(tuntap)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(vde)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(vnet)
-DECLARE_PLUGIN_INIT_FINI_FOR_NET_MODULE(win32)
+PLUGIN_ENTRY_FOR_NET_MODULE(fbsd);
+PLUGIN_ENTRY_FOR_NET_MODULE(linux);
+PLUGIN_ENTRY_FOR_NET_MODULE(null);
+PLUGIN_ENTRY_FOR_NET_MODULE(slirp);
+PLUGIN_ENTRY_FOR_NET_MODULE(socket);
+PLUGIN_ENTRY_FOR_NET_MODULE(tap);
+PLUGIN_ENTRY_FOR_NET_MODULE(tuntap);
+PLUGIN_ENTRY_FOR_NET_MODULE(vde);
+PLUGIN_ENTRY_FOR_NET_MODULE(vnet);
+PLUGIN_ENTRY_FOR_NET_MODULE(win32);
 // USB device plugins
-DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(usb_cbi)
-DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(usb_hid)
-DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(usb_hub)
-DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(usb_msd)
-DECLARE_PLUGIN_INIT_FINI_FOR_USB_MODULE(usb_printer)
+PLUGIN_ENTRY_FOR_MODULE(usb_floppy);
+PLUGIN_ENTRY_FOR_MODULE(usb_hid);
+PLUGIN_ENTRY_FOR_MODULE(usb_hub);
+PLUGIN_ENTRY_FOR_MODULE(usb_msd);
+PLUGIN_ENTRY_FOR_MODULE(usb_printer);
+// disk image plugins
+PLUGIN_ENTRY_FOR_IMG_MODULE(vmware3);
+PLUGIN_ENTRY_FOR_IMG_MODULE(vmware4);
+PLUGIN_ENTRY_FOR_IMG_MODULE(vbox);
+PLUGIN_ENTRY_FOR_IMG_MODULE(vpc);
+PLUGIN_ENTRY_FOR_IMG_MODULE(vvfat);
 
+#endif
 
 #ifdef __cplusplus
 }
